@@ -24,6 +24,7 @@
 #include <sdf/Box.hh>
 #include <sdf/Capsule.hh>
 #include <sdf/Cylinder.hh>
+#include <sdf/Element.hh>
 #include <sdf/Ellipsoid.hh>
 #include <sdf/Gui.hh>
 #include <sdf/Heightmap.hh>
@@ -33,6 +34,7 @@
 #include <sdf/Pbr.hh>
 #include <sdf/Physics.hh>
 #include <sdf/Plane.hh>
+#include <sdf/Polyline.hh>
 #include <sdf/Root.hh>
 #include <sdf/Scene.hh>
 #include <sdf/Sphere.hh>
@@ -564,6 +566,40 @@ TEST(Conversions, GeometryHeightmap)
   EXPECT_DOUBLE_EQ(456.789, newHeightmap->BlendByIndex(0)->FadeDistance());
 }
 
+/////////////////////////////////////////////////
+TEST(Conversions, GeometryPolyline)
+{
+  sdf::Geometry geometry;
+  geometry.SetType(sdf::GeometryType::POLYLINE);
+
+  sdf::Polyline polylineShape;
+  polylineShape.SetHeight(1.23);
+  polylineShape.AddPoint({4.5, 6.7});
+  polylineShape.AddPoint({8.9, 10.11});
+  geometry.SetPolylineShape({polylineShape});
+
+  auto geometryMsg = convert<msgs::Geometry>(geometry);
+  EXPECT_EQ(msgs::Geometry::POLYLINE, geometryMsg.type());
+  ASSERT_EQ(1, geometryMsg.polyline_size());
+  EXPECT_DOUBLE_EQ(1.23, geometryMsg.polyline(0).height());
+  ASSERT_EQ(2, geometryMsg.polyline(0).point_size());
+  EXPECT_DOUBLE_EQ(4.5, geometryMsg.polyline(0).point(0).x());
+  EXPECT_DOUBLE_EQ(6.7, geometryMsg.polyline(0).point(0).y());
+  EXPECT_DOUBLE_EQ(8.9, geometryMsg.polyline(0).point(1).x());
+  EXPECT_DOUBLE_EQ(10.11, geometryMsg.polyline(0).point(1).y());
+
+  auto newGeometry = convert<sdf::Geometry>(geometryMsg);
+  EXPECT_EQ(sdf::GeometryType::POLYLINE, newGeometry.Type());
+  ASSERT_FALSE(newGeometry.PolylineShape().empty());
+  ASSERT_EQ(1u, newGeometry.PolylineShape().size());
+  EXPECT_DOUBLE_EQ(1.23, newGeometry.PolylineShape()[0].Height());
+  ASSERT_EQ(2u, newGeometry.PolylineShape()[0].PointCount());
+  EXPECT_DOUBLE_EQ(4.5, newGeometry.PolylineShape()[0].PointByIndex(0)->X());
+  EXPECT_DOUBLE_EQ(6.7, newGeometry.PolylineShape()[0].PointByIndex(0)->Y());
+  EXPECT_DOUBLE_EQ(8.9, newGeometry.PolylineShape()[0].PointByIndex(1)->X());
+  EXPECT_DOUBLE_EQ(10.11, newGeometry.PolylineShape()[0].PointByIndex(1)->Y());
+}
+
 /////////////////////////////////////////////////
 TEST(Conversions, Inertial)
 {
@@ -720,7 +756,7 @@ TEST(Conversions, Atmosphere)
 }
 
 /////////////////////////////////////////////////
-TEST(CONVERSIONS, MagnetometerSensor)
+TEST(Conversions, MagnetometerSensor)
 {
   sdf::Sensor sensor;
   sensor.SetName("my_sensor");
@@ -760,7 +796,7 @@ TEST(CONVERSIONS, MagnetometerSensor)
 }
 
 /////////////////////////////////////////////////
-TEST(CONVERSIONS, AltimeterSensor)
+TEST(Conversions, AltimeterSensor)
 {
   sdf::Sensor sensor;
   sensor.SetName("my_sensor");
@@ -997,9 +1033,7 @@ TEST(Conversions, ParticleEmitter)
   EXPECT_EQ("topic", header.key());
   EXPECT_EQ("my_topic", header.value(0));
 
-  auto headerScatterRatio = emitterMsg.header().data(1);
-  EXPECT_EQ("particle_scatter_ratio", headerScatterRatio.key());
-  EXPECT_FLOAT_EQ(0.9f, std::stof(headerScatterRatio.value(0)));
+  EXPECT_FLOAT_EQ(0.9f, emitterMsg.particle_scatter_ratio().data());
 
   EXPECT_EQ(math::Pose3d(1, 2, 3, 0, 0, 0), msgs::Convert(emitterMsg.pose()));
 
@@ -1027,3 +1061,98 @@ TEST(Conversions, ParticleEmitter)
   EXPECT_EQ(emitter2.RawPose(), emitter.RawPose());
   EXPECT_FLOAT_EQ(emitter2.ScatterRatio(), emitter.ScatterRatio());
 }
+
+/////////////////////////////////////////////////
+TEST(Conversions, PluginElement)
+{
+  sdf::Root root;
+  root.LoadSdfString("<?xml version='1.0'?><sdf version='1.6'>"
+      "<world name='default'>"
+      "  <plugin filename='plum' name='peach'>"
+      "    <avocado>0.5</avocado>"
+      "  </plugin>"
+      "</world></sdf>");
+
+  auto world = root.WorldByIndex(0);
+  ASSERT_NE(nullptr, world);
+
+  auto worldElem = world->Element();
+  ASSERT_NE(nullptr, worldElem);
+
+  auto pluginElem = worldElem->GetElement("plugin");
+  ASSERT_NE(nullptr, pluginElem);
+
+  auto pluginMsg = convert<msgs::Plugin>(*(pluginElem.get()));
+  EXPECT_EQ("plum", pluginMsg.filename());
+  EXPECT_EQ("peach", pluginMsg.name());
+
+  EXPECT_NE(pluginMsg.innerxml().find("<avocado>0.5</avocado>"),
+      std::string::npos);
+}
+
+/////////////////////////////////////////////////
+TEST(Conversions, Plugin)
+{
+  sdf::Plugin pluginSdf;
+  pluginSdf.SetName("peach");
+  pluginSdf.SetFilename("plum");
+
+  auto content = std::make_shared<sdf::Element>();
+  content->SetName("avocado");
+  content->AddValue("double", "0.5", false, "");
+  pluginSdf.InsertContent(content);
+
+  auto pluginMsg = convert<msgs::Plugin>(pluginSdf);
+  EXPECT_EQ("plum", pluginMsg.filename());
+  EXPECT_EQ("peach", pluginMsg.name());
+
+  EXPECT_NE(pluginMsg.innerxml().find("<avocado>0.5</avocado>"),
+      std::string::npos) << pluginMsg.innerxml();
+}
+
+/////////////////////////////////////////////////
+TEST(Conversions, MsgsPluginToSdf)
+{
+  std::string innerXml ="<test>another_test</test>\n";
+  std::string innerXml2 ="<peanut>butter</peanut>\n";
+
+  msgs::Plugin msgPlugin;
+  msgPlugin.set_name("foo");
+  msgPlugin.set_filename("bar");
+  msgPlugin.set_innerxml(innerXml);
+
+  // Test conversion of a single msgs::Plugin to sdf::Plugin
+  sdf::Plugin sdfPlugin = convert<sdf::Plugin>(msgPlugin);
+  EXPECT_EQ("foo", sdfPlugin.Name());
+  EXPECT_EQ("bar", sdfPlugin.Filename());
+  ASSERT_EQ(1u, sdfPlugin.Contents().size());
+  EXPECT_EQ(innerXml, sdfPlugin.Contents()[0]->ToString(""));
+
+  // Test conversion of a msgs::Plugin_V with 1 plugin to sdf::Plugins
+  msgs::Plugin_V msgsPlugin;
+  msgsPlugin.add_plugins()->CopyFrom(msgPlugin);
+  sdf::Plugins sdfPlugins = convert<sdf::Plugins>(msgsPlugin);
+  ASSERT_EQ(1u, sdfPlugins.size());
+  EXPECT_EQ("foo", sdfPlugins[0].Name());
+  EXPECT_EQ("bar", sdfPlugins[0].Filename());
+  ASSERT_EQ(1u, sdfPlugins[0].Contents().size());
+  EXPECT_EQ(innerXml, sdfPlugins[0].Contents()[0]->ToString(""));
+
+  // Add another plugin the msgs::Plugin_V
+  msgs::Plugin anotherPlugin;
+  anotherPlugin.set_name("sandwich");
+  anotherPlugin.set_filename("time");
+  anotherPlugin.set_innerxml(innerXml + innerXml2);
+  msgsPlugin.add_plugins()->CopyFrom(anotherPlugin);
+  sdfPlugins = convert<sdf::Plugins>(msgsPlugin);
+  ASSERT_EQ(2u, sdfPlugins.size());
+  EXPECT_EQ("foo", sdfPlugins[0].Name());
+  EXPECT_EQ("bar", sdfPlugins[0].Filename());
+  ASSERT_EQ(1u, sdfPlugins[0].Contents().size());
+  EXPECT_EQ(innerXml, sdfPlugins[0].Contents()[0]->ToString(""));
+  EXPECT_EQ("sandwich", sdfPlugins[1].Name());
+  EXPECT_EQ("time", sdfPlugins[1].Filename());
+  ASSERT_EQ(2u, sdfPlugins[1].Contents().size());
+  EXPECT_EQ(innerXml, sdfPlugins[1].Contents()[0]->ToString(""));
+  EXPECT_EQ(innerXml2, sdfPlugins[1].Contents()[1]->ToString(""));
+}
