@@ -25,6 +25,7 @@
 #include <sdf/Model.hh>
 #include <sdf/World.hh>
 
+#include <ignition/math/SphericalCoordinates.hh>
 #include <ignition/common/Profiler.hh>
 
 #include "ignition/gazebo/Events.hh"
@@ -35,14 +36,14 @@
 #include "ignition/gazebo/components/Geometry.hh"
 #include "ignition/gazebo/components/Gravity.hh"
 #include "ignition/gazebo/components/Level.hh"
-#include "ignition/gazebo/components/Model.hh"
-#include "ignition/gazebo/components/Light.hh"
-#include "ignition/gazebo/components/Name.hh"
 #include "ignition/gazebo/components/LevelBuffer.hh"
 #include "ignition/gazebo/components/LevelEntityNames.hh"
+#include "ignition/gazebo/components/Light.hh"
 #include "ignition/gazebo/components/LinearVelocity.hh"
 #include "ignition/gazebo/components/LinearVelocitySeed.hh"
 #include "ignition/gazebo/components/MagneticField.hh"
+#include "ignition/gazebo/components/Model.hh"
+#include "ignition/gazebo/components/Name.hh"
 #include "ignition/gazebo/components/ParentEntity.hh"
 #include "ignition/gazebo/components/Performer.hh"
 #include "ignition/gazebo/components/PerformerLevels.hh"
@@ -50,8 +51,10 @@
 #include "ignition/gazebo/components/PhysicsEnginePlugin.hh"
 #include "ignition/gazebo/components/Pose.hh"
 #include "ignition/gazebo/components/RenderEngineGuiPlugin.hh"
+#include "ignition/gazebo/components/RenderEngineServerHeadless.hh"
 #include "ignition/gazebo/components/RenderEngineServerPlugin.hh"
 #include "ignition/gazebo/components/Scene.hh"
+#include "ignition/gazebo/components/SphericalCoordinates.hh"
 #include "ignition/gazebo/components/Wind.hh"
 #include "ignition/gazebo/components/World.hh"
 
@@ -147,6 +150,10 @@ void LevelManager::ReadLevelPerformerInfo()
       components::RenderEngineServerPlugin(
       this->runner->serverConfig.RenderEngineServer()));
 
+  this->runner->entityCompMgr.CreateComponent(this->worldEntity,
+      components::RenderEngineServerHeadless(
+      this->runner->serverConfig.HeadlessRendering()));
+
   this->runner->entityCompMgr.CreateComponent(this->worldEntity,
       components::RenderEngineGuiPlugin(
       this->runner->serverConfig.RenderEngineGui()));
@@ -180,6 +187,14 @@ void LevelManager::ReadLevelPerformerInfo()
         components::Atmosphere(*this->runner->sdfWorld->Atmosphere()));
   }
 
+  // spherical coordinates
+  if (this->runner->sdfWorld->SphericalCoordinates())
+  {
+    this->runner->entityCompMgr.CreateComponent(this->worldEntity,
+        components::SphericalCoordinates(
+        *this->runner->sdfWorld->SphericalCoordinates()));
+  }
+
   // TODO(anyone) This should probably go somewhere else as it is a global
   // constant.
   const std::string kPluginName{"ignition::gazebo"};
@@ -214,6 +229,9 @@ void LevelManager::ReadLevelPerformerInfo()
   this->ConfigureDefaultLevel();
 
   // Load world plugins.
+  this->runner->EventMgr().Emit<events::LoadSdfPlugins>(this->worldEntity,
+      this->runner->sdfWorld->Plugins());
+  // Deprecate this in Garden
   this->runner->EventMgr().Emit<events::LoadPlugins>(this->worldEntity,
       this->runner->sdfWorld->Element());
 
