@@ -15,6 +15,7 @@
  *
 */
 
+#include "ignition/gazebo/components/CanonicalLink.hh"
 #include "ignition/gazebo/components/Joint.hh"
 #include "ignition/gazebo/components/Link.hh"
 #include "ignition/gazebo/components/Model.hh"
@@ -163,6 +164,14 @@ std::vector<Entity> Model::Links(const EntityComponentManager &_ecm) const
       components::Link());
 }
 
+//////////////////////////////////////////////////
+std::vector<Entity> Model::Models(const EntityComponentManager &_ecm) const
+{
+  return _ecm.EntitiesByComponents(
+      components::ParentEntity(this->dataPtr->id),
+      components::Model());
+}
+
 //////////////////////////////////////////////////
 uint64_t Model::JointCount(const EntityComponentManager &_ecm) const
 {
@@ -194,3 +203,10 @@ void Model::SetWorldPoseCmd(EntityComponentManager &_ecm,
   }
 }
 
+//////////////////////////////////////////////////
+Entity Model::CanonicalLink(const EntityComponentManager &_ecm) const
+{
+  return _ecm.EntityByComponents(
+      components::ParentEntity(this->dataPtr->id),
+      components::CanonicalLink());
+}
