@@ -27,6 +27,7 @@
 #include "ignition/gazebo/components/Altimeter.hh"
 #include "ignition/gazebo/components/AngularVelocity.hh"
 #include "ignition/gazebo/components/Atmosphere.hh"
+#include "ignition/gazebo/components/BoundingBoxCamera.hh"
 #include "ignition/gazebo/components/Camera.hh"
 #include "ignition/gazebo/components/CanonicalLink.hh"
 #include "ignition/gazebo/components/CastShadows.hh"
@@ -57,6 +58,7 @@
 #include "ignition/gazebo/components/Material.hh"
 #include "ignition/gazebo/components/Model.hh"
 #include "ignition/gazebo/components/Name.hh"
+#include "ignition/gazebo/components/NavSat.hh"
 #include "ignition/gazebo/components/ParentLinkName.hh"
 #include "ignition/gazebo/components/ParentEntity.hh"
 #include <ignition/gazebo/components/ParticleEmitter.hh>
@@ -70,6 +72,7 @@
 #include "ignition/gazebo/components/SourceFilePath.hh"
 #include "ignition/gazebo/components/SphericalCoordinates.hh"
 #include "ignition/gazebo/components/Static.hh"
+#include "ignition/gazebo/components/SystemPluginInfo.hh"
 #include "ignition/gazebo/components/ThermalCamera.hh"
 #include "ignition/gazebo/components/ThreadPitch.hh"
 #include "ignition/gazebo/components/Transparency.hh"
@@ -88,15 +91,15 @@ class ignition::gazebo::SdfEntityCreatorPrivate
 
   /// \brief Keep track of new sensors being added, so we load their plugins
   /// only after we have their scoped name.
-  public: std::map<Entity, sdf::ElementPtr> newSensors;
+  public: std::map<Entity, sdf::Plugins> newSensors;
 
   /// \brief Keep track of new models being added, so we load their plugins
   /// only after we have their scoped name.
-  public: std::map<Entity, sdf::ElementPtr> newModels;
+  public: std::map<Entity, sdf::Plugins> newModels;
 
   /// \brief Keep track of new visuals being added, so we load their plugins
   /// only after we have their scoped name.
-  public: std::map<Entity, sdf::ElementPtr> newVisuals;
+  public: std::map<Entity, sdf::Plugins> newVisuals;
 };
 
 using namespace ignition;
@@ -320,8 +323,13 @@ Entity SdfEntityCreator::CreateEntities(const sdf::World *_world)
   this->dataPtr->ecm->CreateComponent(worldEntity,
       components::MagneticField(_world->MagneticField()));
 
-  this->dataPtr->eventManager->Emit<events::LoadPlugins>(worldEntity,
-      _world->Element());
+  this->dataPtr->eventManager->Emit<events::LoadSdfPlugins>(worldEntity,
+      _world->Plugins());
+  for (const sdf::Plugin &p : _world->Plugins())
+  {
+    this->dataPtr->eventManager->Emit<events::LoadPlugins>(worldEntity,
+        p.ToElement());
+  }
 
   // Store the world's SDF DOM to be used when saving the world to file
   this->dataPtr->ecm->CreateComponent(
@@ -338,23 +346,38 @@ Entity SdfEntityCreator::CreateEntities(const sdf::Model *_model)
   auto ent = this->CreateEntities(_model, false);
 
   // Load all model plugins afterwards, so we get scoped name for nested models.
-  for (const auto &[entity, element] : this->dataPtr->newModels)
+  for (const auto &[entity, plugins] : this->dataPtr->newModels)
   {
-    this->dataPtr->eventManager->Emit<events::LoadPlugins>(entity, element);
+    this->dataPtr->eventManager->Emit<events::LoadSdfPlugins>(entity, plugins);
+    for (const sdf::Plugin &p : plugins)
+    {
+      this->dataPtr->eventManager->Emit<events::LoadPlugins>(entity,
+          p.ToElement());
+    }
   }
   this->dataPtr->newModels.clear();
 
   // Load sensor plugins after model, so we get scoped name.
-  for (const auto &[entity, element] : this->dataPtr->newSensors)
+  for (const auto &[entity, plugins] : this->dataPtr->newSensors)
   {
-    this->dataPtr->eventManager->Emit<events::LoadPlugins>(entity, element);
+    this->dataPtr->eventManager->Emit<events::LoadSdfPlugins>(entity, plugins);
+    for (const sdf::Plugin &p : plugins)
+    {
+      this->dataPtr->eventManager->Emit<events::LoadPlugins>(entity,
+          p.ToElement());
+    }
   }
   this->dataPtr->newSensors.clear();
 
   // Load visual plugins after model, so we get scoped name.
-  for (const auto &[entity, element] : this->dataPtr->newVisuals)
+  for (const auto &[entity, plugins] : this->dataPtr->newVisuals)
   {
-    this->dataPtr->eventManager->Emit<events::LoadPlugins>(entity, element);
+    this->dataPtr->eventManager->Emit<events::LoadSdfPlugins>(entity, plugins);
+    for (const sdf::Plugin &p : plugins)
+    {
+      this->dataPtr->eventManager->Emit<events::LoadPlugins>(entity,
+          p.ToElement());
+    }
   }
   this->dataPtr->newVisuals.clear();
 
@@ -381,8 +404,11 @@ Entity SdfEntityCreator::CreateEntities(const sdf::Model *_model,
       modelEntity, components::WindMode(_model->EnableWind()));
   this->dataPtr->ecm->CreateComponent(
       modelEntity, components::SelfCollide(_model->SelfCollide()));
-  this->dataPtr->ecm->CreateComponent(
-      modelEntity, components::SourceFilePath(_model->Element()->FilePath()));
+  if (_model->Element())
+  {
+    this->dataPtr->ecm->CreateComponent(
+        modelEntity, components::SourceFilePath(_model->Element()->FilePath()));
+  }
 
   // NOTE: Pose components of links, visuals, and collisions are expressed in
   // the parent frame until we get frames working.
@@ -449,7 +475,7 @@ Entity SdfEntityCreator::CreateEntities(const sdf::Model *_model,
              << canonicalLinkPair.second << "\n";
     }
   }
-  else if(!isStatic)
+  else if (!isStatic)
   {
     ignerr << "Could not resolve the canonical link for " << _model->Name()
            << "\n";
@@ -461,7 +487,8 @@ Entity SdfEntityCreator::CreateEntities(const sdf::Model *_model,
 
   // Keep track of models so we can load their plugins after loading the entire
   // model and having its full scoped name.
-  this->dataPtr->newModels[modelEntity] = _model->Element();
+  if (!_model->Plugins().empty())
+    this->dataPtr->newModels[modelEntity] = _model->Plugins();
 
   return modelEntity;
 }
@@ -482,8 +509,13 @@ Entity SdfEntityCreator::CreateEntities(const sdf::Actor *_actor)
       components::Name(_actor->Name()));
 
   // Actor plugins
-  this->dataPtr->eventManager->Emit<events::LoadPlugins>(actorEntity,
-      _actor->Element());
+  this->dataPtr->eventManager->Emit<events::LoadSdfPlugins>(actorEntity,
+        _actor->Plugins());
+  for (const sdf::Plugin &p : _actor->Plugins())
+  {
+    this->dataPtr->eventManager->Emit<events::LoadPlugins>(actorEntity,
+        p.ToElement());
+  }
 
   return actorEntity;
 }
@@ -748,9 +780,28 @@ Entity SdfEntityCreator::CreateEntities(const sdf::Visual *_visual)
         components::Material(*_visual->Material()));
   }
 
+  // store the plugin in a component
+  if (!_visual->Plugins().empty())
+  {
+    this->dataPtr->ecm->CreateComponent(visualEntity,
+        components::SystemPluginInfo(
+          convert<msgs::Plugin_V>(_visual->Plugins())));
+  }
+  // Deprecate this in Garden
+  if (_visual->Element())
+  {
+    sdf::ElementPtr pluginElem = _visual->Element()->FindElement("plugin");
+    if (pluginElem)
+    {
+      this->dataPtr->ecm->CreateComponent(visualEntity,
+          components::VisualPlugin(pluginElem));
+    }
+  }
+
   // Keep track of visuals so we can load their plugins after loading the
   // entire model and having its full scoped name.
-  this->dataPtr->newVisuals[visualEntity] = _visual->Element();
+  if (!_visual->Plugins().empty())
+    this->dataPtr->newVisuals[visualEntity] = _visual->Plugins();
 
   return visualEntity;
 }
@@ -856,6 +907,11 @@ Entity SdfEntityCreator::CreateEntities(const sdf::Sensor *_sensor)
     this->dataPtr->ecm->CreateComponent(sensorEntity,
         components::SegmentationCamera(*_sensor));
   }
+  else if (_sensor->Type() == sdf::SensorType::BOUNDINGBOX_CAMERA)
+  {
+    this->dataPtr->ecm->CreateComponent(sensorEntity,
+        components::BoundingBoxCamera(*_sensor));
+  }
   else if (_sensor->Type() == sdf::SensorType::AIR_PRESSURE)
   {
     this->dataPtr->ecm->CreateComponent(sensorEntity,
@@ -876,6 +932,16 @@ Entity SdfEntityCreator::CreateEntities(const sdf::Sensor *_sensor)
     this->dataPtr->ecm->CreateComponent(sensorEntity,
         components::WorldLinearVelocity(math::Vector3d::Zero));
   }
+  else if (_sensor->Type() == sdf::SensorType::GPS ||
+           _sensor->Type() == sdf::SensorType::NAVSAT)
+  {
+    this->dataPtr->ecm->CreateComponent(sensorEntity,
+            components::NavSat(*_sensor));
+
+    // Create components to be filled by physics.
+    this->dataPtr->ecm->CreateComponent(sensorEntity,
+        components::WorldLinearVelocity(math::Vector3d::Zero));
+  }
   else if (_sensor->Type() == sdf::SensorType::IMU)
   {
     this->dataPtr->ecm->CreateComponent(sensorEntity,
@@ -937,7 +1003,8 @@ Entity SdfEntityCreator::CreateEntities(const sdf::Sensor *_sensor)
 
   // Keep track of sensors so we can load their plugins after loading the entire
   // model and having its full scoped name.
-  this->dataPtr->newSensors[sensorEntity] = _sensor->Element();
+  if (!_sensor->Plugins().empty())
+    this->dataPtr->newSensors[sensorEntity] = _sensor->Plugins();
 
   return sensorEntity;
 }
