@@ -61,9 +61,6 @@ SimulationRunner::SimulationRunner(const sdf::World *_world,
   // Keep world name
   this->worldName = _world->Name();
 
-  // Keep system loader so plugins can be loaded at runtime
-  this->systemLoader = _systemLoader;
-
   // Get the physics profile
   // TODO(luca): remove duplicated logic in SdfEntityCreator and LevelManager
   auto physics = _world->PhysicsByIndex(0);
@@ -105,8 +102,38 @@ SimulationRunner::SimulationRunner(const sdf::World *_world,
   // So to get a given RTF, our desired period is:
   //
   // period = step_size / RTF
-  this->updatePeriod = std::chrono::nanoseconds(
-      static_cast<int>(this->stepSize.count() / this->desiredRtf));
+  if (this->desiredRtf < 1e-9)
+  {
+    this->updatePeriod = 0ms;
+  }
+  else
+  {
+    this->updatePeriod = std::chrono::nanoseconds(
+        static_cast<int>(this->stepSize.count() / this->desiredRtf));
+  }
+
+  // World control
+  transport::NodeOptions opts;
+  std::string ns{"/world/" + this->worldName};
+  if (this->networkMgr)
+  {
+    ns = this->networkMgr->Namespace() + ns;
+  }
+
+  auto validNs = transport::TopicUtils::AsValidTopic(ns);
+  if (validNs.empty())
+  {
+    ignerr << "Invalid namespace [" << ns
+           << "], not initializing runner transport." << std::endl;
+    return;
+  }
+  opts.SetNameSpace(validNs);
+
+  this->node = std::make_unique<transport::Node>(opts);
+
+  // Create the system manager
+  this->systemMgr = std::make_unique<SystemManager>(_systemLoader,
+      &this->entityCompMgr, &this->eventMgr, validNs);
 
   this->pauseConn = this->eventMgr.Connect<events::Pause>(
       std::bind(&SimulationRunner::SetPaused, this, std::placeholders::_1));
@@ -114,7 +141,7 @@ SimulationRunner::SimulationRunner(const sdf::World *_world,
   this->stopConn = this->eventMgr.Connect<events::Stop>(
       std::bind(&SimulationRunner::OnStop, this));
 
-  this->loadPluginsConn = this->eventMgr.Connect<events::LoadPlugins>(
+  this->loadPluginsConn = this->eventMgr.Connect<events::LoadSdfPlugins>(
       std::bind(&SimulationRunner::LoadPlugins, this, std::placeholders::_1,
       std::placeholders::_2));
 
@@ -164,9 +191,10 @@ SimulationRunner::SimulationRunner(const sdf::World *_world,
   // Load any additional plugins from the Server Configuration
   this->LoadServerPlugins(this->serverConfig.Plugins());
 
-  // If we have reached this point and no systems have been loaded, then load
-  // a default set of systems.
-  if (this->systems.empty() && this->pendingSystems.empty())
+  // If we have reached this point and no world systems have been loaded, then
+  // load a default set of systems.
+  if (this->systemMgr->TotalByEntity(
+      worldEntity(this->entityCompMgr)).empty())
   {
     ignmsg << "No systems loaded from SDF, loading defaults" << std::endl;
     bool isPlayback = !this->serverConfig.LogPlaybackPath().empty();
@@ -176,25 +204,6 @@ SimulationRunner::SimulationRunner(const sdf::World *_world,
 
   this->LoadLoggingPlugins(this->serverConfig);
 
-  // World control
-  transport::NodeOptions opts;
-  std::string ns{"/world/" + this->worldName};
-  if (this->networkMgr)
-  {
-    ns = this->networkMgr->Namespace() + ns;
-  }
-
-  auto validNs = transport::TopicUtils::AsValidTopic(ns);
-  if (validNs.empty())
-  {
-    ignerr << "Invalid namespace [" << ns
-           << "], not initializing runner transport." << std::endl;
-    return;
-  }
-  opts.SetNameSpace(validNs);
-
-  this->node = std::make_unique<transport::Node>(opts);
-
   // TODO(louise) Combine both messages into one.
   this->node->Advertise("control", &SimulationRunner::OnWorldControl, this);
   this->node->Advertise("control/state", &SimulationRunner::OnWorldControlState,
@@ -448,7 +457,9 @@ void SimulationRunner::AddSystem(const SystemPluginPtr &_system,
       std::optional<Entity> _entity,
       std::optional<std::shared_ptr<const sdf::Element>> _sdf)
 {
-  this->AddSystemImpl(SystemInternal(_system), _entity, _sdf);
+  auto entity = _entity.value_or(worldEntity(this->entityCompMgr));
+  auto sdf = _sdf.value_or(this->sdfWorld->Element());
+  this->systemMgr->AddSystem(_system, entity, sdf);
 }
 
 //////////////////////////////////////////////////
@@ -457,104 +468,57 @@ void SimulationRunner::AddSystem(
       std::optional<Entity> _entity,
       std::optional<std::shared_ptr<const sdf::Element>> _sdf)
 {
-  this->AddSystemImpl(SystemInternal(_system), _entity, _sdf);
-}
-
-//////////////////////////////////////////////////
-void SimulationRunner::AddSystemImpl(
-      SystemInternal _system,
-      std::optional<Entity> _entity,
-      std::optional<std::shared_ptr<const sdf::Element>> _sdf)
-{
-  // Call configure
-  if (_system.configure)
-  {
-    // Default to world entity and SDF
-    auto entity = _entity.has_value() ? _entity.value()
-        : worldEntity(this->entityCompMgr);
-    auto sdf = _sdf.has_value() ? _sdf.value() : this->sdfWorld->Element();
-
-    _system.configure->Configure(
-        entity, sdf,
-        this->entityCompMgr,
-        this->eventMgr);
-  }
-
-  // Update callbacks will be handled later, add to queue
-  std::lock_guard<std::mutex> lock(this->pendingSystemsMutex);
-  this->pendingSystems.push_back(_system);
-}
-
-/////////////////////////////////////////////////
-void SimulationRunner::AddSystemToRunner(SystemInternal _system)
-{
-  this->systems.push_back(_system);
-
-  if (_system.preupdate)
-    this->systemsPreupdate.push_back(_system.preupdate);
-
-  if (_system.update)
-    this->systemsUpdate.push_back(_system.update);
-
-  if (_system.postupdate)
-    this->systemsPostupdate.push_back(_system.postupdate);
+  auto entity = _entity.value_or(worldEntity(this->entityCompMgr));
+  auto sdf = _sdf.value_or(this->sdfWorld->Element());
+  this->systemMgr->AddSystem(_system, entity, sdf);
 }
 
 /////////////////////////////////////////////////
 void SimulationRunner::ProcessSystemQueue()
 {
-  std::lock_guard<std::mutex> lock(this->pendingSystemsMutex);
-  auto pending = this->pendingSystems.size();
+  auto pending = this->systemMgr->PendingCount();
 
-  if (pending > 0)
+  if (0 == pending)
+    return;
+
+  // If additional systems are to be added, stop the existing threads.
+  this->StopWorkerThreads();
+
+  this->systemMgr->ActivatePendingSystems();
+
+  auto threadCount = this->systemMgr->SystemsPostUpdate().size() + 1u;
+
+  igndbg << "Creating PostUpdate worker threads: "
+    << threadCount << std::endl;
+
+  this->postUpdateStartBarrier = std::make_unique<Barrier>(threadCount);
+  this->postUpdateStopBarrier = std::make_unique<Barrier>(threadCount);
+
+  this->postUpdateThreadsRunning = true;
+  int id = 0;
+
+  for (auto &system : this->systemMgr->SystemsPostUpdate())
   {
-    // If additional systems are to be added, stop the existing threads.
-    this->StopWorkerThreads();
-  }
+    igndbg << "Creating postupdate worker thread (" << id << ")" << std::endl;
 
-  for (const auto &system : this->pendingSystems)
-  {
-    this->AddSystemToRunner(system);
-  }
-  this->pendingSystems.clear();
-
-  // If additional systems were added, recreate the worker threads.
-  if (pending > 0)
-  {
-    igndbg << "Creating PostUpdate worker threads: "
-      << this->systemsPostupdate.size() + 1 << std::endl;
-
-    this->postUpdateStartBarrier =
-      std::make_unique<Barrier>(this->systemsPostupdate.size() + 1u);
-    this->postUpdateStopBarrier =
-      std::make_unique<Barrier>(this->systemsPostupdate.size() + 1u);
-
-    this->postUpdateThreadsRunning = true;
-    int id = 0;
-
-    for (auto &system : this->systemsPostupdate)
+    this->postUpdateThreads.push_back(std::thread([&, id]()
     {
-      igndbg << "Creating postupdate worker thread (" << id << ")" << std::endl;
-
-      this->postUpdateThreads.push_back(std::thread([&, id]()
+      std::stringstream ss;
+      ss << "PostUpdateThread: " << id;
+      IGN_PROFILE_THREAD_NAME(ss.str().c_str());
+      while (this->postUpdateThreadsRunning)
       {
-        std::stringstream ss;
-        ss << "PostUpdateThread: " << id;
-        IGN_PROFILE_THREAD_NAME(ss.str().c_str());
-        while (this->postUpdateThreadsRunning)
+        this->postUpdateStartBarrier->Wait();
+        if (this->postUpdateThreadsRunning)
         {
-          this->postUpdateStartBarrier->Wait();
-          if (this->postUpdateThreadsRunning)
-          {
-            system->PostUpdate(this->currentInfo, this->entityCompMgr);
-          }
-          this->postUpdateStopBarrier->Wait();
+          system->PostUpdate(this->currentInfo, this->entityCompMgr);
         }
-        igndbg << "Exiting postupdate worker thread ("
-          << id << ")" << std::endl;
-      }));
-      id++;
-    }
+        this->postUpdateStopBarrier->Wait();
+      }
+      igndbg << "Exiting postupdate worker thread ("
+        << id << ")" << std::endl;
+    }));
+    id++;
   }
 }
 
@@ -570,13 +534,13 @@ void SimulationRunner::UpdateSystems()
 
   {
     IGN_PROFILE("PreUpdate");
-    for (auto& system : this->systemsPreupdate)
+    for (auto& system : this->systemMgr->SystemsPreUpdate())
       system->PreUpdate(this->currentInfo, this->entityCompMgr);
   }
 
   {
     IGN_PROFILE("Update");
-    for (auto& system : this->systemsUpdate)
+    for (auto& system : this->systemMgr->SystemsUpdate())
       system->Update(this->currentInfo, this->entityCompMgr);
   }
 
@@ -843,6 +807,9 @@ void SimulationRunner::Step(const UpdateInfo &_info)
   // so that we can recreate entities with the same name.
   this->ProcessRecreateEntitiesRemove();
 
+  // handle systems that need to be added
+  this->systemMgr->ProcessPendingEntitySystems();
+
   // Update all the systems.
   this->UpdateSystems();
 
@@ -892,23 +859,9 @@ void SimulationRunner::Step(const UpdateInfo &_info)
 
 //////////////////////////////////////////////////
 void SimulationRunner::LoadPlugin(const Entity _entity,
-                                  const std::string &_fname,
-                                  const std::string &_name,
-                                  const sdf::ElementPtr &_sdf)
+                                  const sdf::Plugin &_plugin)
 {
-  std::optional<SystemPluginPtr> system;
-  {
-    std::lock_guard<std::mutex> lock(this->systemLoaderMutex);
-    system = this->systemLoader->LoadPlugin(_fname, _name, _sdf);
-  }
-
-  // System correctly loaded from library
-  if (system)
-  {
-    this->AddSystem(system.value(), _entity, _sdf);
-    igndbg << "Loaded system [" << _name
-           << "] for entity [" << _entity << "]" << std::endl;
-  }
+  this->systemMgr->LoadPlugin(_entity, _plugin);
 }
 
 //////////////////////////////////////////////////
@@ -988,7 +941,7 @@ void SimulationRunner::LoadServerPlugins(
 
     if (kNullEntity != entity)
     {
-      this->LoadPlugin(entity, plugin.Filename(), plugin.Name(), plugin.Sdf());
+      this->LoadPlugin(entity, plugin.Plugin());
     }
   }
 }
@@ -1020,23 +973,18 @@ void SimulationRunner::LoadLoggingPlugins(const ServerConfig &_config)
 
 //////////////////////////////////////////////////
 void SimulationRunner::LoadPlugins(const Entity _entity,
-    const sdf::ElementPtr &_sdf)
+    const sdf::Plugins &_plugins)
 {
-  sdf::ElementPtr pluginElem = _sdf->FindElement("plugin");
-  while (pluginElem)
+  for (const sdf::Plugin &plugin : _plugins)
   {
-    auto filename = pluginElem->Get<std::string>("filename");
-    auto name = pluginElem->Get<std::string>("name");
     // No error message for the 'else' case of the following 'if' statement
     // because SDF create a default <plugin> element even if it's not
     // specified. An error message would result in spamming
     // the console.
-    if (filename != "__default__" && name != "__default__")
+    if (plugin.Filename() != "__default__" && plugin.Name() != "__default__")
     {
-      this->LoadPlugin(_entity, filename, name, pluginElem);
+      this->LoadPlugin(_entity, plugin);
     }
-
-    pluginElem = pluginElem->GetNextElement("plugin");
   }
 }
 
@@ -1078,8 +1026,7 @@ size_t SimulationRunner::EntityCount() const
 /////////////////////////////////////////////////
 size_t SimulationRunner::SystemCount() const
 {
-  std::lock_guard<std::mutex> lock(this->pendingSystemsMutex);
-  return this->systems.size() + this->pendingSystems.size();
+  return this->systemMgr->TotalCount();
 }
 
 /////////////////////////////////////////////////
