@@ -21,6 +21,7 @@
 #include <ignition/common/Console.hh>
 #include <ignition/common/Util.hh>
 #include <ignition/transport/Node.hh>
+#include <ignition/utilities/ExtraTestMacros.hh>
 #include <sdf/Box.hh>
 #include <sdf/Capsule.hh>
 #include <sdf/Cylinder.hh>
@@ -1073,7 +1074,7 @@ TEST_P(SimulationRunnerTest, Time)
   EXPECT_EQ(0u, runner.CurrentInfo().iterations);
   EXPECT_EQ(0ms, runner.CurrentInfo().simTime);
   EXPECT_EQ(0ms, runner.CurrentInfo().dt);
-  EXPECT_EQ(1ms, runner.UpdatePeriod());
+  EXPECT_EQ(0ms, runner.UpdatePeriod());
   EXPECT_EQ(1ms, runner.StepSize());
 
   runner.SetPaused(false);
@@ -1086,7 +1087,7 @@ TEST_P(SimulationRunnerTest, Time)
   EXPECT_EQ(100u, runner.CurrentInfo().iterations);
   EXPECT_EQ(100ms, runner.CurrentInfo().simTime);
   EXPECT_EQ(1ms, runner.CurrentInfo().dt);
-  EXPECT_EQ(1ms, runner.UpdatePeriod());
+  EXPECT_EQ(0ms, runner.UpdatePeriod());
   EXPECT_EQ(1ms, runner.StepSize());
 
   int sleep = 0;
@@ -1107,7 +1108,7 @@ TEST_P(SimulationRunnerTest, Time)
   EXPECT_EQ(200u, runner.CurrentInfo().iterations);
   EXPECT_EQ(300ms, runner.CurrentInfo().simTime);
   EXPECT_EQ(2ms, runner.CurrentInfo().dt);
-  EXPECT_EQ(1ms, runner.UpdatePeriod());
+  EXPECT_EQ(0ms, runner.UpdatePeriod());
   EXPECT_EQ(2ms, runner.StepSize());
 
   sleep = 0;
@@ -1127,7 +1128,7 @@ TEST_P(SimulationRunnerTest, Time)
   EXPECT_EQ(200u, runner.CurrentInfo().iterations);
   EXPECT_EQ(300ms, runner.CurrentInfo().simTime);
   EXPECT_EQ(2ms, runner.CurrentInfo().dt);
-  EXPECT_EQ(1ms, runner.UpdatePeriod());
+  EXPECT_EQ(0ms, runner.UpdatePeriod());
   EXPECT_EQ(2ms, runner.StepSize());
 
   // Verify info published to /clock topic
@@ -1145,7 +1146,7 @@ TEST_P(SimulationRunnerTest, Time)
   EXPECT_EQ(500ms, runner.CurrentInfo().simTime)
     << runner.CurrentInfo().simTime.count();
   EXPECT_EQ(2ms, runner.CurrentInfo().dt);
-  EXPECT_EQ(1ms, runner.UpdatePeriod());
+  EXPECT_EQ(0ms, runner.UpdatePeriod());
   EXPECT_EQ(2ms, runner.StepSize());
 
   sleep = 0;
@@ -1183,7 +1184,8 @@ TEST_P(SimulationRunnerTest, Time)
 }
 
 /////////////////////////////////////////////////
-TEST_P(SimulationRunnerTest, LoadPlugins)
+// See https://github.com/ignitionrobotics/ign-gazebo/issues/1175
+TEST_P(SimulationRunnerTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LoadPlugins) )
 {
   // Load SDF file
   sdf::Root root;
@@ -1232,6 +1234,18 @@ TEST_P(SimulationRunnerTest, LoadPlugins)
       });
   EXPECT_NE(kNullEntity, sensorId);
 
+  // Get visual entity
+  Entity visualId{kNullEntity};
+  runner.EntityCompMgr().Each<ignition::gazebo::components::Visual>([&](
+      const ignition::gazebo::Entity &_entity,
+      const ignition::gazebo::components::Visual *_visual)->bool
+      {
+        EXPECT_NE(nullptr, _visual);
+        visualId = _entity;
+        return true;
+      });
+  EXPECT_NE(kNullEntity, visualId);
+
   // Check component registered by world plugin
   std::string worldComponentName{"WorldPluginComponent"};
   auto worldComponentId = ignition::common::hash64(worldComponentName);
@@ -1256,6 +1270,14 @@ TEST_P(SimulationRunnerTest, LoadPlugins)
   EXPECT_TRUE(runner.EntityCompMgr().EntityHasComponentType(sensorId,
       sensorComponentId));
 
+  // Check component registered by visual plugin
+  std::string visualComponentName{"VisualPluginComponent"};
+  auto visualComponentId = ignition::common::hash64(visualComponentName);
+
+  EXPECT_TRUE(runner.EntityCompMgr().HasComponentType(visualComponentId));
+  EXPECT_TRUE(runner.EntityCompMgr().EntityHasComponentType(visualId,
+      visualComponentId));
+
   // Clang re-registers components between tests. If we don't unregister them
   // beforehand, the new plugin tries to create a storage type from a previous
   // plugin, causing a crash.
@@ -1266,11 +1288,13 @@ TEST_P(SimulationRunnerTest, LoadPlugins)
     components::Factory::Instance()->Unregister(worldComponentId);
     components::Factory::Instance()->Unregister(modelComponentId);
     components::Factory::Instance()->Unregister(sensorComponentId);
+    components::Factory::Instance()->Unregister(visualComponentId);
   #endif
 }
 
 /////////////////////////////////////////////////
-TEST_P(SimulationRunnerTest, LoadServerNoPlugins)
+TEST_P(SimulationRunnerTest,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(LoadServerNoPlugins) )
 {
   sdf::Root rootWithout;
   rootWithout.Load(common::joinPaths(PROJECT_SOURCE_PATH,
@@ -1292,7 +1316,8 @@ TEST_P(SimulationRunnerTest, LoadServerNoPlugins)
 }
 
 /////////////////////////////////////////////////
-TEST_P(SimulationRunnerTest, LoadServerConfigPlugins)
+TEST_P(SimulationRunnerTest,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(LoadServerConfigPlugins) )
 {
   sdf::Root rootWithout;
   rootWithout.Load(common::joinPaths(PROJECT_SOURCE_PATH,
@@ -1392,7 +1417,8 @@ TEST_P(SimulationRunnerTest, LoadServerConfigPlugins)
 }
 
 /////////////////////////////////////////////////
-TEST_P(SimulationRunnerTest, LoadPluginsDefault)
+TEST_P(SimulationRunnerTest,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(LoadPluginsDefault) )
 {
   sdf::Root rootWithout;
   rootWithout.Load(common::joinPaths(PROJECT_SOURCE_PATH,
@@ -1413,7 +1439,8 @@ TEST_P(SimulationRunnerTest, LoadPluginsDefault)
 }
 
 /////////////////////////////////////////////////
-TEST_P(SimulationRunnerTest, LoadPluginsEvent)
+TEST_P(SimulationRunnerTest,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(LoadPluginsEvent) )
 {
   // Load SDF file without plugins
   sdf::Root rootWithout;
@@ -1457,8 +1484,8 @@ TEST_P(SimulationRunnerTest, LoadPluginsEvent)
   ASSERT_EQ(1u, rootWith.WorldCount());
 
   // Emit plugin loading event
-  runner.EventMgr().Emit<events::LoadPlugins>(boxEntity,
-      rootWith.WorldByIndex(0)->ModelByIndex(0)->Element());
+  runner.EventMgr().Emit<events::LoadSdfPlugins>(boxEntity,
+      rootWith.WorldByIndex(0)->ModelByIndex(0)->Plugins());
 
   // Check component registered by model plugin
   EXPECT_TRUE(runner.EntityCompMgr().HasComponentType(componentId))
@@ -1467,8 +1494,8 @@ TEST_P(SimulationRunnerTest, LoadPluginsEvent)
       componentId)) << componentId;
 
   // Emit plugin loading event again
-  runner.EventMgr().Emit<events::LoadPlugins>(sphereEntity,
-      rootWith.WorldByIndex(0)->ModelByIndex(0)->Element());
+  runner.EventMgr().Emit<events::LoadSdfPlugins>(sphereEntity,
+      rootWith.WorldByIndex(0)->ModelByIndex(0)->Plugins());
 
   // Check component for the other model
   EXPECT_TRUE(runner.EntityCompMgr().HasComponentType(componentId))
@@ -1495,8 +1522,8 @@ TEST_P(SimulationRunnerTest, LoadPluginsEvent)
   EXPECT_FALSE(runner.EntityCompMgr().HasEntity(sphereEntity));
 
   // Emit plugin loading event after all previous instances have been removed
-  runner.EventMgr().Emit<events::LoadPlugins>(cylinderEntity,
-      rootWith.WorldByIndex(0)->ModelByIndex(0)->Element());
+  runner.EventMgr().Emit<events::LoadSdfPlugins>(cylinderEntity,
+      rootWith.WorldByIndex(0)->ModelByIndex(0)->Plugins());
 
   // Check component for the other model
   EXPECT_TRUE(runner.EntityCompMgr().HasComponentType(componentId))
@@ -1505,6 +1532,30 @@ TEST_P(SimulationRunnerTest, LoadPluginsEvent)
       componentId)) << componentId;
 }
 
+/////////////////////////////////////////////////
+TEST_P(SimulationRunnerTest,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(LoadOnlyModelPlugin) )
+{
+  sdf::Root rootWithout;
+  rootWithout.Load(common::joinPaths(PROJECT_SOURCE_PATH,
+      "test", "worlds", "model_plugin_only.sdf"));
+  ASSERT_EQ(1u, rootWithout.WorldCount());
+
+  // ServerConfig will fall back to environment variable
+  auto config = common::joinPaths(PROJECT_SOURCE_PATH,
+    "test", "worlds", "server_valid2.config");
+  ASSERT_EQ(true, common::setenv(gazebo::kServerConfigPathEnv, config));
+  ServerConfig serverConfig;
+
+  // Create simulation runner
+  auto systemLoader = std::make_shared<SystemLoader>();
+  SimulationRunner runner(rootWithout.WorldByIndex(0), systemLoader,
+      serverConfig);
+
+  // 1 model plugin from SDF and 2 world plugins from config
+  ASSERT_EQ(3u, runner.SystemCount());
+}
+
 /////////////////////////////////////////////////
 TEST_P(SimulationRunnerTest, GuiInfo)
 {
