@@ -20,21 +20,21 @@
 #include <sdf/Root.hh>
 #include <sdf/World.hh>
 
-#include <gz/common/Filesystem.hh>
-#include "gz/sim/System.hh"
-#include "gz/sim/SystemLoader.hh"
+#include <ignition/common/Filesystem.hh>
+#include "ignition/gazebo/System.hh"
+#include "ignition/gazebo/SystemLoader.hh"
 
-#include "gz/sim/test_config.hh"  // NOLINT(build/include)
+#include "ignition/gazebo/test_config.hh"  // NOLINT(build/include)
 
-using namespace gz;
+using namespace ignition;
 
 /////////////////////////////////////////////////
 TEST(SystemLoader, Constructor)
 {
-  gz::sim::SystemLoader sm;
+  gazebo::SystemLoader sm;
 
   // Add test plugin to path (referenced in config)
-  auto testBuildPath = common::joinPaths(
+  auto testBuildPath = ignition::common::joinPaths(
       std::string(PROJECT_BINARY_PATH), "lib");
   sm.AddSystemPluginPath(testBuildPath);
 
@@ -43,7 +43,7 @@ TEST(SystemLoader, Constructor)
       "<world name='default'>"
       "<plugin filename='libignition-gazebo") +
       IGNITION_GAZEBO_MAJOR_VERSION_STR + "-physics-system.so' "
-      "name='gz::sim::systems::Physics'></plugin>"
+      "name='ignition::gazebo::systems::Physics'></plugin>"
       "</world></sdf>");
 
   auto worldElem = root.WorldByIndex(0)->Element();
@@ -51,7 +51,9 @@ TEST(SystemLoader, Constructor)
     sdf::ElementPtr pluginElem = worldElem->GetElement("plugin");
     while (pluginElem)
     {
-      auto system = sm.LoadPlugin(pluginElem);
+      sdf::Plugin plugin;
+      plugin.Load(pluginElem);
+      auto system = sm.LoadPlugin(plugin);
       ASSERT_TRUE(system.has_value());
       pluginElem = pluginElem->GetNextElement("plugin");
     }
@@ -60,8 +62,8 @@ TEST(SystemLoader, Constructor)
 
 TEST(SystemLoader, EmptyNames)
 {
-  gz::sim::SystemLoader sm;
-  sdf::ElementPtr element;
-  auto system = sm.LoadPlugin("", "", element);
+  gazebo::SystemLoader sm;
+  sdf::Plugin plugin;
+  auto system = sm.LoadPlugin(plugin);
   ASSERT_FALSE(system.has_value());
 }
