@@ -17,11 +17,8 @@
 
 #include "QuickStartHandler.hh"
 
-#include <gz/common/Console.hh>
-
-using namespace gz;
-using namespace gz::sim;
-using namespace ignition::gazebo::gui;
+using namespace ignition;
+using namespace gazebo::gui;
 
 /////////////////////////////////////////////////
 QString QuickStartHandler::WorldsPath() const
