@@ -16,8 +16,8 @@
 */
 
 #include <iostream>
+#include <list>
 #include <regex>
-#include <QColorDialog>
 #include <ignition/common/Console.hh>
 #include <ignition/common/Profiler.hh>
 #include <ignition/gui/Application.hh>
@@ -59,7 +59,9 @@
 #include "ignition/gazebo/components/SelfCollide.hh"
 #include "ignition/gazebo/components/Sensor.hh"
 #include "ignition/gazebo/components/SourceFilePath.hh"
+#include "ignition/gazebo/components/SphericalCoordinates.hh"
 #include "ignition/gazebo/components/Static.hh"
+#include "ignition/gazebo/components/SystemPluginInfo.hh"
 #include "ignition/gazebo/components/ThreadPitch.hh"
 #include "ignition/gazebo/components/Transparency.hh"
 #include "ignition/gazebo/components/Visual.hh"
@@ -71,6 +73,7 @@
 
 #include "ComponentInspector.hh"
 #include "Pose3d.hh"
+#include "SystemPluginInfo.hh"
 
 namespace ignition::gazebo
 {
@@ -112,6 +115,9 @@ namespace ignition::gazebo
 
     /// \brief Handles all components displayed as a 3D pose.
     public: std::unique_ptr<inspector::Pose3d> pose3d;
+
+    /// \brief Handles all system info components.
+    public: std::unique_ptr<inspector::SystemPluginInfo> systemInfo;
   };
 }
 
@@ -311,28 +317,48 @@ void ignition::gazebo::setData(QStandardItem *_item,
   _item->setData(QString("Material"),
       ComponentsModel::RoleNames().key("dataType"));
   _item->setData(QList({
-    QVariant(_data.Ambient().R() * 255),
-    QVariant(_data.Ambient().G() * 255),
-    QVariant(_data.Ambient().B() * 255),
-    QVariant(_data.Ambient().A() * 255),
-    QVariant(_data.Diffuse().R() * 255),
-    QVariant(_data.Diffuse().G() * 255),
-    QVariant(_data.Diffuse().B() * 255),
-    QVariant(_data.Diffuse().A() * 255),
-    QVariant(_data.Specular().R() * 255),
-    QVariant(_data.Specular().G() * 255),
-    QVariant(_data.Specular().B() * 255),
-    QVariant(_data.Specular().A() * 255),
-    QVariant(_data.Emissive().R() * 255),
-    QVariant(_data.Emissive().G() * 255),
-    QVariant(_data.Emissive().B() * 255),
-    QVariant(_data.Emissive().A() * 255)
+    QVariant(_data.Ambient().R()),
+    QVariant(_data.Ambient().G()),
+    QVariant(_data.Ambient().B()),
+    QVariant(_data.Ambient().A()),
+    QVariant(_data.Diffuse().R()),
+    QVariant(_data.Diffuse().G()),
+    QVariant(_data.Diffuse().B()),
+    QVariant(_data.Diffuse().A()),
+    QVariant(_data.Specular().R()),
+    QVariant(_data.Specular().G()),
+    QVariant(_data.Specular().B()),
+    QVariant(_data.Specular().A()),
+    QVariant(_data.Emissive().R()),
+    QVariant(_data.Emissive().G()),
+    QVariant(_data.Emissive().B()),
+    QVariant(_data.Emissive().A())
   }), ComponentsModel::RoleNames().key("data"));
 
   // TODO(anyone) Only shows colors of material,
   // need to add others (e.g., pbr)
 }
 
+//////////////////////////////////////////////////
+template<>
+void ignition::gazebo::setData(QStandardItem *_item,
+    const math::SphericalCoordinates &_data)
+{
+  if (nullptr == _item)
+    return;
+
+  _item->setData(QString("SphericalCoordinates"),
+      ComponentsModel::RoleNames().key("dataType"));
+  _item->setData(QList({
+    QVariant(QString::fromStdString(math::SphericalCoordinates::Convert(
+        _data.Surface()))),
+    QVariant(_data.LatitudeReference().Degree()),
+    QVariant(_data.LongitudeReference().Degree()),
+    QVariant(_data.ElevationReference()),
+    QVariant(_data.HeadingOffset().Degree()),
+  }), ComponentsModel::RoleNames().key("data"));
+}
+
 //////////////////////////////////////////////////
 void ignition::gazebo::setUnit(QStandardItem *_item, const std::string &_unit)
 {
@@ -453,6 +479,8 @@ void ComponentInspector::LoadConfig(const tinyxml2::XMLElement *)
 
   // Type-specific handlers
   this->dataPtr->pose3d = std::make_unique<inspector::Pose3d>(this);
+  this->dataPtr->systemInfo =
+      std::make_unique<inspector::SystemPluginInfo>(this);
 }
 
 //////////////////////////////////////////////////
@@ -777,6 +805,13 @@ void ComponentInspector::Update(const UpdateInfo &,
       if (comp)
         setData(item, comp->Data());
     }
+    else if (typeId == components::SphericalCoordinates::typeId)
+    {
+      auto comp = _ecm.Component<components::SphericalCoordinates>(
+          this->dataPtr->entity);
+      if (comp)
+        setData(item, comp->Data());
+    }
     else if (typeId == components::ThreadPitch::typeId)
     {
       auto comp = _ecm.Component<components::ThreadPitch>(
@@ -866,18 +901,25 @@ void ComponentInspector::Update(const UpdateInfo &,
     }
   }
 
-  // Remove components no longer present
+  // Remove components no longer present - list items to remove
+  std::list<ignition::gazebo::ComponentTypeId> itemsToRemove;
   for (auto itemIt : this->dataPtr->componentsModel.items)
   {
     auto typeId = itemIt.first;
     if (componentTypes.find(typeId) == componentTypes.end())
     {
-      QMetaObject::invokeMethod(&this->dataPtr->componentsModel,
-          "RemoveComponentType",
-          Qt::QueuedConnection,
-          Q_ARG(ignition::gazebo::ComponentTypeId, typeId));
+      itemsToRemove.push_back(typeId);
     }
   }
+
+  // Remove components in list
+  for (auto typeId : itemsToRemove)
+  {
+    QMetaObject::invokeMethod(&this->dataPtr->componentsModel,
+        "RemoveComponentType",
+        Qt::QueuedConnection,
+        Q_ARG(ignition::gazebo::ComponentTypeId, typeId));
+  }
 }
 
 /////////////////////////////////////////////////
@@ -1085,55 +1127,8 @@ void ComponentInspector::OnMaterialColor(
   double _rDiffuse, double _gDiffuse, double _bDiffuse, double _aDiffuse,
   double _rSpecular, double _gSpecular, double _bSpecular, double _aSpecular,
   double _rEmissive, double _gEmissive, double _bEmissive, double _aEmissive,
-  QString _type, QColor _currColor)
+  QString /*_type*/, QColor /*_currColor*/)
 {
-  // when type is not empty, open qt color dialog
-  std::string type = _type.toStdString();
-  if (!type.empty())
-  {
-    QColor newColor = QColorDialog::getColor(
-        _currColor, nullptr, "Pick a color",
-        {QColorDialog::DontUseNativeDialog, QColorDialog::ShowAlphaChannel});
-
-    // returns if the user hits cancel
-    if (!newColor.isValid())
-      return;
-
-    if (type == "ambient")
-    {
-      _rAmbient = newColor.red();
-      _gAmbient = newColor.green();
-      _bAmbient = newColor.blue();
-      _aAmbient = newColor.alpha();
-    }
-    else if (type == "diffuse")
-    {
-      _rDiffuse = newColor.red();
-      _gDiffuse = newColor.green();
-      _bDiffuse = newColor.blue();
-      _aDiffuse = newColor.alpha();
-    }
-    else if (type == "specular")
-    {
-      _rSpecular = newColor.red();
-      _gSpecular = newColor.green();
-      _bSpecular = newColor.blue();
-      _aSpecular = newColor.alpha();
-    }
-    else if (type == "emissive")
-    {
-      _rEmissive = newColor.red();
-      _gEmissive = newColor.green();
-      _bEmissive = newColor.blue();
-      _aEmissive = newColor.alpha();
-    }
-    else
-    {
-      ignerr << "Invalid material type: " << type << std::endl;
-      return;
-    }
-  }
-
   std::function<void(const ignition::msgs::Boolean &, const bool)> cb =
       [](const ignition::msgs::Boolean &/*_rep*/, const bool _result)
   {
@@ -1146,17 +1141,13 @@ void ComponentInspector::OnMaterialColor(
   req.set_id(this->dataPtr->entity);
 
   msgs::Set(req.mutable_material()->mutable_ambient(),
-    math::Color(_rAmbient / 255.0, _gAmbient / 255.0,
-      _bAmbient / 255.0, _aAmbient / 255.0));
+    math::Color(_rAmbient, _gAmbient, _bAmbient, _aAmbient));
   msgs::Set(req.mutable_material()->mutable_diffuse(),
-    math::Color(_rDiffuse / 255.0, _gDiffuse / 255.0,
-      _bDiffuse / 255.0, _aDiffuse / 255.0));
+    math::Color(_rDiffuse, _gDiffuse, _bDiffuse, _aDiffuse));
   msgs::Set(req.mutable_material()->mutable_specular(),
-    math::Color(_rSpecular / 255.0, _gSpecular / 255.0,
-      _bSpecular / 255.0, _aSpecular / 255.0));
+    math::Color(_rSpecular, _gSpecular, _bSpecular, _aSpecular));
   msgs::Set(req.mutable_material()->mutable_emissive(),
-    math::Color(_rEmissive / 255.0, _gEmissive / 255.0,
-      _bEmissive / 255.0, _aEmissive / 255.0));
+    math::Color(_rEmissive, _gEmissive, _bEmissive, _aEmissive));
 
   auto materialCmdService = "/world/" + this->dataPtr->worldName
       + "/visual_config";
@@ -1169,6 +1160,44 @@ void ComponentInspector::OnMaterialColor(
   this->dataPtr->node.Request(materialCmdService, req, cb);
 }
 
+/////////////////////////////////////////////////
+void ComponentInspector::OnSphericalCoordinates(QString _surface,
+    double _latitude, double _longitude, double _elevation,
+    double _heading)
+{
+  if (_surface != QString("EARTH_WGS84"))
+  {
+    ignerr << "Surface [" << _surface.toStdString() << "] not supported."
+           << std::endl;
+    return;
+  }
+
+  std::function<void(const msgs::Boolean &, const bool)> cb =
+      [](const msgs::Boolean &/*_rep*/, const bool _result)
+  {
+    if (!_result)
+      ignerr << "Error setting spherical coordinates." << std::endl;
+  };
+
+  msgs::SphericalCoordinates req;
+  req.set_surface_model(msgs::SphericalCoordinates::EARTH_WGS84);
+  req.set_latitude_deg(_latitude);
+  req.set_longitude_deg(_longitude);
+  req.set_elevation(_elevation);
+  req.set_heading_deg(_heading);
+
+  auto sphericalCoordsCmdService = "/world/" + this->dataPtr->worldName
+      + "/set_spherical_coordinates";
+  sphericalCoordsCmdService =
+      transport::TopicUtils::AsValidTopic(sphericalCoordsCmdService);
+  if (sphericalCoordsCmdService.empty())
+  {
+    ignerr << "Invalid spherical coordinates service" << std::endl;
+    return;
+  }
+  this->dataPtr->node.Request(sphericalCoordsCmdService, req, cb);
+}
+
 /////////////////////////////////////////////////
 bool ComponentInspector::NestedModel() const
 {
