@@ -19,7 +19,9 @@
 #include <cstdio>
 #include <cstdlib>
 
+#include <fstream>
 #include <string>
+#include <ignition/common/Filesystem.hh>
 #include <ignition/common/Util.hh>
 #include <ignition/utilities/ExtraTestMacros.hh>
 
@@ -29,6 +31,8 @@ static const std::string kBinPath(PROJECT_BINARY_PATH);
 
 static const std::string kIgnCommand(
     std::string(BREW_RUBY) + std::string(IGN_PATH) + "/ign gazebo -s ");
+static const std::string kIgnModelCommand(
+    std::string(BREW_RUBY) + std::string(IGN_PATH) + "/ign model ");
 
 /////////////////////////////////////////////////
 std::string customExecStr(std::string _cmd)
@@ -55,7 +59,8 @@ std::string customExecStr(std::string _cmd)
 }
 
 /////////////////////////////////////////////////
-TEST(CmdLine, Server)
+// See https://github.com/ignitionrobotics/ign-gazebo/issues/1175
+TEST(CmdLine, IGN_UTILS_TEST_DISABLED_ON_WIN32(Server))
 {
   std::string cmd = kIgnCommand + " -r -v 4 --iterations 5 " +
     std::string(PROJECT_SOURCE_PATH) + "/test/worlds/plugins.sdf";
@@ -87,7 +92,7 @@ TEST(CmdLine, Server)
 }
 
 /////////////////////////////////////////////////
-TEST(CmdLine, CachedFuelWorld)
+TEST(CmdLine, IGN_UTILS_TEST_DISABLED_ON_WIN32(CachedFuelWorld))
 {
   std::string projectPath = std::string(PROJECT_SOURCE_PATH) + "/test/worlds";
   ignition::common::setenv("IGN_FUEL_CACHE_PATH", projectPath.c_str());
@@ -101,7 +106,7 @@ TEST(CmdLine, CachedFuelWorld)
 }
 
 /////////////////////////////////////////////////
-TEST(CmdLine, GazeboServer)
+TEST(CmdLine, IGN_UTILS_TEST_DISABLED_ON_WIN32(GazeboServer))
 {
   std::string cmd = kIgnCommand + " -r -v 4 --iterations 5 " +
     std::string(PROJECT_SOURCE_PATH) + "/test/worlds/plugins.sdf";
@@ -118,7 +123,7 @@ TEST(CmdLine, GazeboServer)
 }
 
 /////////////////////////////////////////////////
-TEST(CmdLine, Gazebo)
+TEST(CmdLine, IGN_UTILS_TEST_DISABLED_ON_WIN32(Gazebo))
 {
   std::string cmd = kIgnCommand + " -r -v 4 --iterations 5 " +
     std::string(PROJECT_SOURCE_PATH) + "/test/worlds/plugins.sdf";
@@ -135,7 +140,7 @@ TEST(CmdLine, Gazebo)
 }
 
 /////////////////////////////////////////////////
-TEST(CmdLine, ResourcePath)
+TEST(CmdLine, IGN_UTILS_TEST_DISABLED_ON_WIN32(ResourcePath))
 {
   std::string cmd = kIgnCommand + " -s -r -v 4 --iterations 1 plugins.sdf";
 
@@ -160,3 +165,65 @@ TEST(CmdLine, ResourcePath)
   EXPECT_EQ(output.find("Unable to find file plugins.sdf"), std::string::npos)
       << output;
 }
+
+//////////////////////////////////////////////////
+/// \brief Check --help message and bash completion script for consistent flags
+TEST(CmdLine, IGN_UTILS_TEST_DISABLED_ON_WIN32(GazeboHelpVsCompletionFlags))
+{
+  // Flags in help message
+  std::string helpOutput = customExecStr(kIgnCommand + " gazebo --help");
+
+  // Call the output function in the bash completion script
+  std::string scriptPath = ignition::common::joinPaths(
+    std::string(PROJECT_SOURCE_PATH),
+    "src", "cmd", "gazebo.bash_completion.sh");
+
+  // Equivalent to:
+  // sh -c "bash -c \". /path/to/gazebo.bash_completion.sh; _gz_sim_flags\""
+  std::string cmd = "bash -c \". " + scriptPath + "; _gz_sim_flags\"";
+  std::string scriptOutput = customExecStr(cmd);
+
+  // Tokenize script output
+  std::istringstream iss(scriptOutput);
+  std::vector<std::string> flags((std::istream_iterator<std::string>(iss)),
+    std::istream_iterator<std::string>());
+
+  EXPECT_GT(flags.size(), 0u);
+
+  // Match each flag in script output with help message
+  for (const auto &flag : flags)
+  {
+    EXPECT_NE(std::string::npos, helpOutput.find(flag)) << helpOutput;
+  }
+}
+
+//////////////////////////////////////////////////
+/// \brief Check --help message and bash completion script for consistent flags
+TEST(CmdLine, IGN_UTILS_TEST_DISABLED_ON_WIN32(ModelHelpVsCompletionFlags))
+{
+  // Flags in help message
+  std::string helpOutput = customExecStr(kIgnModelCommand + " --help");
+
+  // Call the output function in the bash completion script
+  std::string scriptPath = ignition::common::joinPaths(
+    std::string(PROJECT_SOURCE_PATH),
+    "src", "cmd", "model.bash_completion.sh");
+
+  // Equivalent to:
+  // sh -c "bash -c \". /path/to/model.bash_completion.sh; _gz_model_flags\""
+  std::string cmd = "bash -c \". " + scriptPath + "; _gz_model_flags\"";
+  std::string scriptOutput = customExecStr(cmd);
+
+  // Tokenize script output
+  std::istringstream iss(scriptOutput);
+  std::vector<std::string> flags((std::istream_iterator<std::string>(iss)),
+    std::istream_iterator<std::string>());
+
+  EXPECT_GT(flags.size(), 0u);
+
+  // Match each flag in script output with help message
+  for (const auto &flag : flags)
+  {
+    EXPECT_NE(std::string::npos, helpOutput.find(flag)) << helpOutput;
+  }
+}
