@@ -26,6 +26,7 @@
 #include <set>
 #include <list>
 
+#include <ignition/common/Time.hh>
 #include <ignition/common/Console.hh>
 #include <ignition/common/Filesystem.hh>
 #include <ignition/common/Profiler.hh>
@@ -54,7 +55,6 @@
 #include "ignition/gazebo/components/Material.hh"
 #include "ignition/gazebo/components/Model.hh"
 #include "ignition/gazebo/components/Name.hh"
-#include "ignition/gazebo/components/Pose.hh"
 #include "ignition/gazebo/components/SourceFilePath.hh"
 #include "ignition/gazebo/components/Visual.hh"
 #include "ignition/gazebo/components/World.hh"
@@ -158,6 +158,12 @@ class ignition::gazebo::systems::LogRecordPrivate
 
   /// \brief List of saved models if record with resources is enabled.
   public: std::set<std::string> savedModels;
+
+  /// \brief Time period between state recording
+  public: std::chrono::steady_clock::duration recordPeriod{0};
+
+  /// \brief Last time states are recorded
+  public: std::chrono::steady_clock::duration lastRecordSimTime{0};
 };
 
 bool LogRecordPrivate::started{false};
@@ -210,6 +216,11 @@ void LogRecord::Configure(const Entity &_entity,
   this->dataPtr->SetRecordResources(_sdf->Get<bool>("record_resources",
     false).first);
 
+  this->dataPtr->recordPeriod =
+    std::chrono::duration_cast<std::chrono::steady_clock::duration>(
+    std::chrono::duration<double>(
+    _sdf->Get<double>("record_period", 0.0).first));
+
   this->dataPtr->compress = _sdf->Get<bool>("compress", false).first;
   this->dataPtr->cmpPath = _sdf->Get<std::string>("compress_path", "").first;
 
@@ -320,13 +331,8 @@ bool LogRecordPrivate::Start(const std::string &_logPath,
   ignmsg << "Recording to log file [" << dbPath << "]" << std::endl;
 
   // Add default topics if no topics were specified.
-  std::string dynPoseTopic = "/world/" + this->worldName +
-    "/dynamic_pose/info";
-
-  igndbg << "Recording default topic[" << dynPoseTopic << "].\n";
   igndbg << "Recording default topic[" << sdfTopic << "].\n";
   igndbg << "Recording default topic[" << stateTopic << "].\n";
-  this->recorder.AddTopic(dynPoseTopic);
   this->recorder.AddTopic(sdfTopic);
   this->recorder.AddTopic(stateTopic);
 
@@ -696,16 +702,34 @@ void LogRecord::PostUpdate(const UpdateInfo &_info,
     }
   }
 
+  bool record = true;
+  if (this->dataPtr->recordPeriod > std::chrono::steady_clock::duration::zero())
+  {
+    if (_ecm.HasOneTimeComponentChanges() ||
+        (_info.simTime - this->dataPtr->lastRecordSimTime) >=
+        this->dataPtr->recordPeriod)
+    {
+      this->dataPtr->lastRecordSimTime = _info.simTime;
+    }
+    else
+    {
+      record = false;
+    }
+  }
+
   // TODO(louise) Use the SceneBroadcaster's topic once that publishes
   // the changed state
   // \todo(anyone) A potential enhancement here is have a keyframe mechanism
   // to store complete state periodically, and then store incremental from
   // that. It would reduce some of the compute on replaying
   // (especially in tools like plotting or seeking through logs).
-  msgs::SerializedStateMap stateMsg;
-  _ecm.ChangedState(stateMsg);
-  if (!stateMsg.entities().empty())
-    this->dataPtr->statePub.Publish(stateMsg);
+  if (record)
+  {
+    msgs::SerializedStateMap stateMsg;
+    _ecm.ChangedState(stateMsg);
+    if (!stateMsg.entities().empty())
+      this->dataPtr->statePub.Publish(stateMsg);
+  }
 
   // If there are new models loaded, save meshes and textures
   if (this->dataPtr->RecordResources() && _ecm.HasNewEntities())
