@@ -17,7 +17,7 @@
 
 #include "PosePublisher.hh"
 
-#include <gz/msgs/pose.pb.h>
+#include <ignition/msgs/pose.pb.h>
 
 #include <stack>
 #include <string>
@@ -28,31 +28,30 @@
 
 #include <sdf/Joint.hh>
 
-#include <gz/common/Profiler.hh>
-#include <gz/common/Time.hh>
-#include <gz/math/Pose3.hh>
-#include <gz/plugin/Register.hh>
-#include <gz/transport/Node.hh>
+#include <ignition/common/Profiler.hh>
+#include <ignition/math/Pose3.hh>
+#include <ignition/plugin/Register.hh>
+#include <ignition/transport/Node.hh>
 
-#include "gz/sim/Util.hh"
-#include "gz/sim/components/CanonicalLink.hh"
-#include "gz/sim/components/ChildLinkName.hh"
-#include "gz/sim/components/Collision.hh"
-#include "gz/sim/components/Joint.hh"
-#include "gz/sim/components/JointType.hh"
-#include "gz/sim/components/Link.hh"
-#include "gz/sim/components/Model.hh"
-#include "gz/sim/components/Name.hh"
-#include "gz/sim/components/ParentEntity.hh"
-#include "gz/sim/components/ParentLinkName.hh"
-#include "gz/sim/components/Pose.hh"
-#include "gz/sim/components/Sensor.hh"
-#include "gz/sim/components/Visual.hh"
-#include "gz/sim/Conversions.hh"
-#include "gz/sim/Model.hh"
+#include "ignition/gazebo/Util.hh"
+#include "ignition/gazebo/components/CanonicalLink.hh"
+#include "ignition/gazebo/components/ChildLinkName.hh"
+#include "ignition/gazebo/components/Collision.hh"
+#include "ignition/gazebo/components/Joint.hh"
+#include "ignition/gazebo/components/JointType.hh"
+#include "ignition/gazebo/components/Link.hh"
+#include "ignition/gazebo/components/Model.hh"
+#include "ignition/gazebo/components/Name.hh"
+#include "ignition/gazebo/components/ParentEntity.hh"
+#include "ignition/gazebo/components/ParentLinkName.hh"
+#include "ignition/gazebo/components/Pose.hh"
+#include "ignition/gazebo/components/Sensor.hh"
+#include "ignition/gazebo/components/Visual.hh"
+#include "ignition/gazebo/Conversions.hh"
+#include "ignition/gazebo/Model.hh"
 
-using namespace gz;
-using namespace gz::sim;
+using namespace ignition;
+using namespace gazebo;
 using namespace systems;
 
 /// \brief Private data class for PosePublisher
@@ -112,6 +111,9 @@ class ignition::gazebo::systems::PosePublisherPrivate
   /// \brief True to publish nested model pose
   public: bool publishNestedModelPose = false;
 
+  /// \brief True to publish model pose
+  public: bool publishModelPose = false;
+
   /// \brief Frequency of pose publications in Hz. A negative frequency
   /// publishes as fast as possible (i.e, at the rate of the simulation step)
   public: double updateFrequency = -1;
@@ -154,12 +156,12 @@ class ignition::gazebo::systems::PosePublisherPrivate
   /// \brief A variable that gets populated with poses. This also here as a
   /// member variable to avoid repeated memory allocations and improve
   /// performance.
-  public: msgs::Pose poseMsg;
+  public: ignition::msgs::Pose poseMsg;
 
   /// \brief A variable that gets populated with poses. This also here as a
   /// member variable to avoid repeated memory allocations and improve
   /// performance.
-  public: msgs::Pose_V poseVMsg;
+  public: ignition::msgs::Pose_V poseVMsg;
 
   /// \brief True to publish a vector of poses. False to publish individual pose
   /// msgs.
@@ -198,6 +200,12 @@ void PosePublisher::Configure(const Entity &_entity,
     _sdf->Get<bool>("publish_nested_model_pose",
         this->dataPtr->publishNestedModelPose).first;
 
+  // for backward compatibility, publish_model_pose will be set to the
+  // same value as publish_nested_model_pose if it is not specified.
+  this->dataPtr->publishModelPose =
+    _sdf->Get<bool>("publish_model_pose",
+        this->dataPtr->publishNestedModelPose).first;
+
   this->dataPtr->publishVisualPose =
     _sdf->Get<bool>("publish_visual_pose",
         this->dataPtr->publishVisualPose).first;
@@ -243,28 +251,35 @@ void PosePublisher::Configure(const Entity &_entity,
     _sdf->Get<bool>("use_pose_vector_msg", this->dataPtr->usePoseV).first;
 
   std::string poseTopic = scopedName(_entity, _ecm) + "/pose";
+  poseTopic = transport::TopicUtils::AsValidTopic(poseTopic);
+  if (poseTopic.empty())
+  {
+    poseTopic = "/pose";
+    ignerr << "Empty pose topic generated for pose_publisher system. "
+           << "Setting to " << poseTopic << std::endl;
+  }
   std::string staticPoseTopic = poseTopic + "_static";
 
   if (this->dataPtr->usePoseV)
   {
     this->dataPtr->posePub =
-      this->dataPtr->node.Advertise<msgs::Pose_V>(poseTopic);
+      this->dataPtr->node.Advertise<ignition::msgs::Pose_V>(poseTopic);
 
     if (this->dataPtr->staticPosePublisher)
     {
       this->dataPtr->poseStaticPub =
-          this->dataPtr->node.Advertise<msgs::Pose_V>(
+          this->dataPtr->node.Advertise<ignition::msgs::Pose_V>(
           staticPoseTopic);
     }
   }
   else
   {
     this->dataPtr->posePub =
-      this->dataPtr->node.Advertise<msgs::Pose>(poseTopic);
+      this->dataPtr->node.Advertise<ignition::msgs::Pose>(poseTopic);
     if (this->dataPtr->staticPosePublisher)
     {
       this->dataPtr->poseStaticPub =
-          this->dataPtr->node.Advertise<msgs::Pose>(
+          this->dataPtr->node.Advertise<ignition::msgs::Pose>(
           staticPoseTopic);
     }
   }
@@ -365,18 +380,36 @@ void PosePublisherPrivate::InitializeEntitiesToPublish(
     visited.push_back(entity);
 
     auto link = _ecm.Component<components::Link>(entity);
-    auto nestedModel = _ecm.Component<components::Model>(entity);
     auto visual = _ecm.Component<components::Visual>(entity);
     auto collision = _ecm.Component<components::Collision>(entity);
     auto sensor = _ecm.Component<components::Sensor>(entity);
     auto joint = _ecm.Component<components::Joint>(entity);
 
+    auto isModel = _ecm.Component<components::Model>(entity);
+    auto parent = _ecm.Component<components::ParentEntity>(entity);
+
     bool fillPose = (link && this->publishLinkPose) ||
-        (nestedModel && this->publishNestedModelPose) ||
         (visual && this->publishVisualPose) ||
         (collision && this->publishCollisionPose) ||
         (sensor && this->publishSensorPose);
 
+    // for backward compatibility, top level model pose will be published
+    // if publishNestedModelPose is set to true unless the user explicity
+    // disables this by setting publishModelPose to false
+    if (isModel)
+    {
+      if (parent)
+      {
+        auto nestedModel = _ecm.Component<components::Model>(parent->Data());
+        if (nestedModel)
+          fillPose = this->publishNestedModelPose;
+      }
+      if (!fillPose)
+      {
+        fillPose = this->publishNestedModelPose && this->publishModelPose;
+      }
+    }
+
     if (fillPose)
     {
       std::string frame;
@@ -387,7 +420,6 @@ void PosePublisherPrivate::InitializeEntitiesToPublish(
       childFrame =
         removeParentScope(scopedName(entity, _ecm, "::", false), "::");
 
-      auto parent = _ecm.Component<components::ParentEntity>(entity);
       if (parent)
       {
         auto parentName = _ecm.Component<components::Name>(parent->Data());
@@ -498,7 +530,7 @@ void PosePublisherPrivate::PublishPoses(
   IGN_PROFILE("PosePublisher::PublishPoses");
 
   // publish poses
-  msgs::Pose *msg = nullptr;
+  ignition::msgs::Pose *msg = nullptr;
   if (this->usePoseV)
     this->poseVMsg.Clear();
 
@@ -555,9 +587,5 @@ IGNITION_ADD_PLUGIN(PosePublisher,
                     PosePublisher::ISystemConfigure,
                     PosePublisher::ISystemPostUpdate)
 
-IGNITION_ADD_PLUGIN_ALIAS(PosePublisher,
-                          "gz::sim::systems::PosePublisher")
-
-// TODO(CH3): Deprecated, remove on version 8
 IGNITION_ADD_PLUGIN_ALIAS(PosePublisher,
                           "ignition::gazebo::systems::PosePublisher")
