@@ -20,6 +20,7 @@
 #include <map>
 #include <set>
 #include <unordered_map>
+#include <unordered_set>
 #include <utility>
 #include <vector>
 
@@ -31,17 +32,27 @@
 #include <ignition/math/Helpers.hh>
 
 #include <ignition/rendering/Scene.hh>
+#include <ignition/sensors/BoundingBoxCameraSensor.hh>
 #include <ignition/sensors/CameraSensor.hh>
+#include <ignition/sensors/DepthCameraSensor.hh>
+#include <ignition/sensors/GpuLidarSensor.hh>
 #include <ignition/sensors/RenderingSensor.hh>
+#include <ignition/sensors/RgbdCameraSensor.hh>
 #include <ignition/sensors/ThermalCameraSensor.hh>
+#include <ignition/sensors/SegmentationCameraSensor.hh>
 #include <ignition/sensors/Manager.hh>
 
 #include "ignition/gazebo/components/Atmosphere.hh"
+#include "ignition/gazebo/components/BatterySoC.hh"
+#include "ignition/gazebo/components/BoundingBoxCamera.hh"
 #include "ignition/gazebo/components/Camera.hh"
 #include "ignition/gazebo/components/DepthCamera.hh"
 #include "ignition/gazebo/components/GpuLidar.hh"
+#include "ignition/gazebo/components/ParentEntity.hh"
+#include "ignition/gazebo/components/RenderEngineServerHeadless.hh"
 #include "ignition/gazebo/components/RenderEngineServerPlugin.hh"
 #include "ignition/gazebo/components/RgbdCamera.hh"
+#include "ignition/gazebo/components/SegmentationCamera.hh"
 #include "ignition/gazebo/components/ThermalCamera.hh"
 #include "ignition/gazebo/components/World.hh"
 #include "ignition/gazebo/Events.hh"
@@ -85,7 +96,6 @@ class ignition::gazebo::systems::SensorsPrivate
   /// \brief Keep track of cameras, in case we need to handle stereo cameras.
   /// Key: Camera's parent scoped name
   /// Value: Pointer to camera
-  // TODO(anyone) Remove element when sensor is deleted
   public: std::map<std::string, sensors::CameraSensor *> cameras;
 
   /// \brief Maps gazebo entity to its matching sensor ID
@@ -175,11 +185,39 @@ class ignition::gazebo::systems::SensorsPrivate
   /// \brief Stop the rendering thread
   public: void Stop();
 
+  /// \brief Update battery state of sensors in model
+  /// \param[in] _ecm Entity component manager
+  public: void UpdateBatteryState(const EntityComponentManager &_ecm);
+
+  /// \brief Check if sensor has subscribers
+  /// \param[in] _sensor Sensor to check
+  /// \return True if the sensor has subscribers, false otherwise
+  public: bool HasConnections(sensors::RenderingSensor *_sensor) const;
+
   /// \brief Use to optionally set the background color.
   public: std::optional<math::Color> backgroundColor;
 
   /// \brief Use to optionally set the ambient light.
   public: std::optional<math::Color> ambientLight;
+
+  /// \brief A map between model entity ids in the ECM to its battery state
+  /// True means has charge, false means drained
+  public: std::unordered_map<Entity, bool> modelBatteryState;
+
+  /// \brief A map between model entity ids in the ECM to whether its battery
+  /// state has changed.
+  /// True means has charge, false means drained
+  public: std::unordered_map<Entity, bool> modelBatteryStateChanged;
+
+  /// \brief A map of sensor ids to their active state
+  public: std::map<sensors::SensorId, bool> sensorStateChanged;
+
+  /// \brief Disable sensors if parent model's battery is drained
+  /// Affects sensors that are in nested models
+  public: bool disableOnDrainedBattery = false;
+
+  /// \brief Mutex to protect access to sensorStateChanged
+  public: std::mutex sensorStateMutex;
 };
 
 //////////////////////////////////////////////////
@@ -206,6 +244,7 @@ void SensorsPrivate::WaitForInit()
         this->renderUtil.SetAmbientLight(*this->ambientLight);
       this->renderUtil.Init();
       this->scene = this->renderUtil.Scene();
+      this->scene->SetCameraPassCountPerGpuFlush(6u);
       this->initialized = true;
     }
 
@@ -236,9 +275,25 @@ void SensorsPrivate::RunOnce()
     this->renderUtil.Update();
   }
 
-
   if (!this->activeSensors.empty())
   {
+    // disable sensors that are out of battery or re-enable sensors that are
+    // being charged
+    if (this->disableOnDrainedBattery)
+    {
+      std::unique_lock<std::mutex> lock2(this->sensorStateMutex);
+      for (const auto &sensorIt : this->sensorStateChanged)
+      {
+        sensors::Sensor *s =
+            this->sensorManager.Sensor(sensorIt.first);
+        if (s)
+        {
+          s->SetActive(sensorIt.second);
+        }
+      }
+      this->sensorStateChanged.clear();
+    }
+
     this->sensorMaskMutex.lock();
     // Check the active sensors against masked sensors.
     //
@@ -261,6 +316,7 @@ void SensorsPrivate::RunOnce()
     {
       IGN_PROFILE("PreRender");
       this->eventManager->Emit<events::PreRender>();
+      this->scene->SetTime(this->updateTime);
       // Update the scene graph manually to improve performance
       // We only need to do this once per frame It is important to call
       // sensors::RenderingSensor::SetManualSceneUpdate and set it to true
@@ -268,10 +324,41 @@ void SensorsPrivate::RunOnce()
       this->scene->PreRender();
     }
 
+    // disable sensors that have no subscribers to prevent doing unnecessary
+    // work
+    std::unordered_set<sensors::RenderingSensor *> tmpDisabledSensors;
+    this->sensorMaskMutex.lock();
+    for (auto id : this->sensorIds)
+    {
+      sensors::Sensor *s = this->sensorManager.Sensor(id);
+      auto rs = dynamic_cast<sensors::RenderingSensor *>(s);
+      if (rs->IsActive() && !this->HasConnections(rs))
+      {
+        rs->SetActive(false);
+        tmpDisabledSensors.insert(rs);
+      }
+    }
+    this->sensorMaskMutex.unlock();
+
     {
       // publish data
       IGN_PROFILE("RunOnce");
       this->sensorManager.RunOnce(this->updateTime);
+    }
+
+    // re-enble sensors
+    for (auto &rs : tmpDisabledSensors)
+    {
+      rs->SetActive(true);
+    }
+
+    {
+      IGN_PROFILE("PostRender");
+      // Update the scene graph manually to improve performance
+      // We only need to do this once per frame It is important to call
+      // sensors::RenderingSensor::SetManualSceneUpdate and set it to true
+      // so we don't waste cycles doing one scene graph update per sensor
+      this->scene->PostRender();
       this->eventManager->Emit<events::PostRender>();
     }
 
@@ -355,6 +442,17 @@ void Sensors::RemoveSensor(const Entity &_entity)
         this->dataPtr->activeSensors.erase(activeSensorIt);
       }
     }
+
+    // update cameras list
+    for (auto &it : this->dataPtr->cameras)
+    {
+      if (it.second->Id() == idIter->second)
+      {
+        this->dataPtr->cameras.erase(it.first);
+        break;
+      }
+    }
+
     this->dataPtr->sensorIds.erase(idIter->second);
     this->dataPtr->sensorManager.Remove(idIter->second);
     this->dataPtr->entityToIdMap.erase(idIter);
@@ -384,6 +482,11 @@ void Sensors::Configure(const Entity &/*_id*/,
   std::string engineName =
       _sdf->Get<std::string>("render_engine", "ogre2").first;
 
+  // get whether or not to disable sensor when model battery is drained
+  this->dataPtr->disableOnDrainedBattery =
+      _sdf->Get<bool>("disable_on_drained_battery",
+     this->dataPtr-> disableOnDrainedBattery).first;
+
   // Get the background color, if specified.
   if (_sdf->HasElement("background_color"))
     this->dataPtr->backgroundColor = _sdf->Get<math::Color>("background_color");
@@ -396,8 +499,9 @@ void Sensors::Configure(const Entity &/*_id*/,
   this->dataPtr->renderUtil.SetEnableSensors(true,
       std::bind(&Sensors::CreateSensor, this,
       std::placeholders::_1, std::placeholders::_2, std::placeholders::_3));
-    this->dataPtr->renderUtil.SetRemoveSensorCb(
+  this->dataPtr->renderUtil.SetRemoveSensorCb(
       std::bind(&Sensors::RemoveSensor, this, std::placeholders::_1));
+  this->dataPtr->renderUtil.SetEventManager(&_eventMgr);
 
   // parse sensor-specific data
   auto worldEntity = _ecm.EntityByComponents(components::World());
@@ -420,6 +524,15 @@ void Sensors::Configure(const Entity &/*_id*/,
     {
       this->dataPtr->renderUtil.SetEngineName(renderEngineServerComp->Data());
     }
+
+    // Set headless mode if specified from command line
+    auto renderEngineServerHeadlessComp =
+      _ecm.Component<components::RenderEngineServerHeadless>(worldEntity);
+    if (renderEngineServerHeadlessComp)
+    {
+      this->dataPtr->renderUtil.SetHeadlessRendering(
+        renderEngineServerHeadlessComp->Data());
+    }
   }
 
   this->dataPtr->eventManager = &_eventMgr;
@@ -457,7 +570,6 @@ void Sensors::PostUpdate(const UpdateInfo &_info,
         << "s]. System may not work properly." << std::endl;
   }
 
-
   {
     std::unique_lock<std::mutex> lock(this->dataPtr->renderMutex);
     if (!this->dataPtr->initialized &&
@@ -465,7 +577,10 @@ void Sensors::PostUpdate(const UpdateInfo &_info,
          _ecm.HasComponentType(components::DepthCamera::typeId) ||
          _ecm.HasComponentType(components::GpuLidar::typeId) ||
          _ecm.HasComponentType(components::RgbdCamera::typeId) ||
-         _ecm.HasComponentType(components::ThermalCamera::typeId))) {
+         _ecm.HasComponentType(components::ThermalCamera::typeId) ||
+         _ecm.HasComponentType(components::SegmentationCamera::typeId) ||
+         _ecm.HasComponentType(components::BoundingBoxCamera::typeId)))
+    {
       igndbg << "Initialization needed" << std::endl;
       this->dataPtr->doInit = true;
       this->dataPtr->renderCv.notify_one();
@@ -510,6 +625,9 @@ void Sensors::PostUpdate(const UpdateInfo &_info,
     if (!activeSensors.empty() ||
         this->dataPtr->renderUtil.PendingSensors() > 0)
     {
+      if (this->dataPtr->disableOnDrainedBattery)
+        this->dataPtr->UpdateBatteryState(_ecm);
+
       std::unique_lock<std::mutex> lock(this->dataPtr->renderMutex);
       this->dataPtr->renderCv.wait(lock, [this] {
         return !this->dataPtr->running || !this->dataPtr->updateAvailable; });
@@ -527,6 +645,67 @@ void Sensors::PostUpdate(const UpdateInfo &_info,
   }
 }
 
+
+//////////////////////////////////////////////////
+void SensorsPrivate::UpdateBatteryState(const EntityComponentManager &_ecm)
+{
+  // Battery state
+  _ecm.Each<components::BatterySoC>(
+      [&](const Entity & _entity, const components::BatterySoC *_bat)
+      {
+        bool hasCharge = _bat->Data() > 0;
+        auto stateIt =
+          this->modelBatteryState.find(_ecm.ParentEntity(_entity));
+        if (stateIt != this->modelBatteryState.end())
+        {
+          // detect a change in battery charge state
+          if (stateIt->second != hasCharge)
+          {
+            this->modelBatteryStateChanged[_ecm.ParentEntity(_entity)] =
+                hasCharge;
+          }
+        }
+        this->modelBatteryState[_ecm.ParentEntity(_entity)] = hasCharge;
+        return true;
+      });
+
+  // disable sensor if parent model is out of battery or re-enable sensor
+  // if battery is charging
+  for (const auto & modelIt : this->modelBatteryStateChanged)
+  {
+    // check if sensor is part of this model
+    for (const auto & sensorIt : this->entityToIdMap)
+    {
+      // parent link
+      auto parentLinkComp =
+          _ecm.Component<components::ParentEntity>(sensorIt.first);
+      if (!parentLinkComp)
+        continue;
+
+      // parent model
+      auto parentModelComp = _ecm.Component<components::ParentEntity>(
+          parentLinkComp->Data());
+      if (!parentModelComp)
+        continue;
+
+      // keep going up the tree in case sensor is in a nested model
+      while (parentModelComp)
+      {
+        auto parentEnt = parentModelComp->Data();
+        if (parentEnt == modelIt.first)
+        {
+          std::unique_lock<std::mutex> lock(this->sensorStateMutex);
+          // sensor is part of model - update its active state
+          this->sensorStateChanged[sensorIt.second] = modelIt.second;
+          break;
+        }
+        parentModelComp = _ecm.Component<components::ParentEntity>(parentEnt);
+      }
+    }
+  }
+  this->modelBatteryStateChanged.clear();
+}
+
 //////////////////////////////////////////////////
 std::string Sensors::CreateSensor(const Entity &_entity,
     const sdf::Sensor &_sdf, const std::string &_parentName)
@@ -538,24 +717,57 @@ std::string Sensors::CreateSensor(const Entity &_entity,
   }
 
   // Create within ign-sensors
-  auto sensorId = this->dataPtr->sensorManager.CreateSensor(_sdf);
-  auto sensor = this->dataPtr->sensorManager.Sensor(sensorId);
+  sensors::Sensor *sensor{nullptr};
+  if (_sdf.Type() == sdf::SensorType::CAMERA)
+  {
+    sensor = this->dataPtr->sensorManager.CreateSensor<
+      sensors::CameraSensor>(_sdf);
+  }
+  else if (_sdf.Type() == sdf::SensorType::DEPTH_CAMERA)
+  {
+    sensor = this->dataPtr->sensorManager.CreateSensor<
+      sensors::DepthCameraSensor>(_sdf);
+  }
+  else if (_sdf.Type() == sdf::SensorType::GPU_LIDAR)
+  {
+    sensor = this->dataPtr->sensorManager.CreateSensor<
+      sensors::GpuLidarSensor>(_sdf);
+  }
+  else if (_sdf.Type() == sdf::SensorType::RGBD_CAMERA)
+  {
+    sensor = this->dataPtr->sensorManager.CreateSensor<
+      sensors::RgbdCameraSensor>(_sdf);
+  }
+  else if (_sdf.Type() == sdf::SensorType::THERMAL_CAMERA)
+  {
+    sensor = this->dataPtr->sensorManager.CreateSensor<
+      sensors::ThermalCameraSensor>(_sdf);
+  }
+  else if (_sdf.Type() == sdf::SensorType::BOUNDINGBOX_CAMERA)
+  {
+    sensor = this->dataPtr->sensorManager.CreateSensor<
+      sensors::BoundingBoxCameraSensor>(_sdf);
+  }
+  else if (_sdf.Type() == sdf::SensorType::SEGMENTATION_CAMERA)
+  {
+    sensor = this->dataPtr->sensorManager.CreateSensor<
+      sensors::SegmentationCameraSensor>(_sdf);
+  }
 
-  // Add to sensorID -> entity map
-  this->dataPtr->entityToIdMap.insert({_entity, sensorId});
-
-  if (nullptr == sensor || sensors::NO_SENSOR == sensor->Id())
+  if (nullptr == sensor)
   {
     ignerr << "Failed to create sensor [" << _sdf.Name()
-           << "]" << std::endl;
+           << "]." << std::endl;
     return std::string();
   }
 
+  // Store sensor ID
+  auto sensorId = sensor->Id();
+  this->dataPtr->entityToIdMap.insert({_entity, sensorId});
   this->dataPtr->sensorIds.insert(sensorId);
 
   // Set the scene so it can create the rendering sensor
-  auto renderingSensor =
-      dynamic_cast<sensors::RenderingSensor *>(sensor);
+  auto renderingSensor = dynamic_cast<sensors::RenderingSensor *>(sensor);
   renderingSensor->SetScene(this->dataPtr->scene);
   renderingSensor->SetParent(_parentName);
   renderingSensor->SetManualSceneUpdate(true);
@@ -629,6 +841,54 @@ std::string Sensors::CreateSensor(const Entity &_entity,
   return sensor->Name();
 }
 
+//////////////////////////////////////////////////
+bool SensorsPrivate::HasConnections(sensors::RenderingSensor *_sensor) const
+{
+  if (!_sensor)
+    return true;
+
+  // \todo(iche033) Remove this function once a virtual
+  // sensors::RenderingSensor::HasConnections function is available
+  {
+    auto s = dynamic_cast<sensors::RgbdCameraSensor *>(_sensor);
+    if (s)
+      return s->HasConnections();
+  }
+  {
+    auto s = dynamic_cast<sensors::DepthCameraSensor *>(_sensor);
+    if (s)
+      return s->HasConnections();
+  }
+  {
+    auto s = dynamic_cast<sensors::GpuLidarSensor *>(_sensor);
+    if (s)
+      return s->HasConnections();
+  }
+  {
+    auto s = dynamic_cast<sensors::SegmentationCameraSensor *>(_sensor);
+    if (s)
+      return s->HasConnections();
+  }
+  {
+    auto s = dynamic_cast<sensors::BoundingBoxCameraSensor *>(_sensor);
+    if (s)
+      return s->HasConnections();
+  }
+  {
+    auto s = dynamic_cast<sensors::ThermalCameraSensor *>(_sensor);
+    if (s)
+      return s->HasConnections();
+  }
+  {
+    auto s = dynamic_cast<sensors::CameraSensor *>(_sensor);
+    if (s)
+      return s->HasConnections();
+  }
+  ignwarn << "Unable to check connection count for sensor: " << _sensor->Name()
+          << ". Unknown sensor type." << std::endl;
+  return true;
+}
+
 IGNITION_ADD_PLUGIN(Sensors, System,
   Sensors::ISystemConfigure,
   Sensors::ISystemUpdate,
