@@ -17,16 +17,29 @@
 
 #include "UserCommands.hh"
 
+#ifdef _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4251)
+#endif
+
 #include <google/protobuf/message.h>
+
+#ifdef _MSC_VER
+#pragma warning(pop)
+#endif
+
 #include <ignition/msgs/boolean.pb.h>
 #include <ignition/msgs/entity_factory.pb.h>
 #include <ignition/msgs/light.pb.h>
 #include <ignition/msgs/pose.pb.h>
+#include <ignition/msgs/pose_v.pb.h>
 #include <ignition/msgs/physics.pb.h>
 #include <ignition/msgs/visual.pb.h>
+#include <ignition/msgs/wheel_slip_parameters_cmd.pb.h>
 
 #include <string>
 #include <utility>
+#include <unordered_set>
 #include <vector>
 
 #include <ignition/math/SphericalCoordinates.hh>
@@ -42,6 +55,8 @@
 
 #include "ignition/common/Profiler.hh"
 
+#include "ignition/gazebo/components/Collision.hh"
+#include "ignition/gazebo/components/Joint.hh"
 #include "ignition/gazebo/components/Light.hh"
 #include "ignition/gazebo/components/LightCmd.hh"
 #include "ignition/gazebo/components/Link.hh"
@@ -52,15 +67,19 @@
 #include "ignition/gazebo/components/PoseCmd.hh"
 #include "ignition/gazebo/components/PhysicsCmd.hh"
 #include "ignition/gazebo/components/SphericalCoordinates.hh"
+#include "ignition/gazebo/components/Visual.hh"
 #include "ignition/gazebo/components/World.hh"
 #include "ignition/gazebo/Conversions.hh"
 #include "ignition/gazebo/EntityComponentManager.hh"
+#include "ignition/gazebo/Model.hh"
 #include "ignition/gazebo/SdfEntityCreator.hh"
+#include "ignition/gazebo/Util.hh"
 #include "ignition/gazebo/World.hh"
 #include "ignition/gazebo/components/ContactSensorData.hh"
 #include "ignition/gazebo/components/ContactSensor.hh"
 #include "ignition/gazebo/components/Sensor.hh"
 #include "ignition/gazebo/components/VisualCmd.hh"
+#include "ignition/gazebo/components/WheelSlipCmd.hh"
 
 using namespace ignition;
 using namespace gazebo;
@@ -73,74 +92,6 @@ namespace gazebo
 inline namespace IGNITION_GAZEBO_VERSION_NAMESPACE {
 namespace systems
 {
-
-/// \brief Helper function to get an entity from an entity message.
-///
-/// \TODO(anyone) Move to Util.hh and generalize for all entities,
-/// not only top level
-///
-/// The message is used as follows:
-///
-///     if id not null
-///       use id
-///     else if name not null and type not null
-///       use name + type
-///     else
-///       error
-///     end
-/// \param[in] _ecm Entity component manager
-/// \param[in] _msg Entity message
-/// \return Entity ID, or kNullEntity if a matching entity couldn't be
-/// found.
-Entity topLevelEntityFromMessage(const EntityComponentManager &_ecm,
-    const msgs::Entity &_msg)
-{
-  if (_msg.id() != kNullEntity)
-  {
-    return _msg.id();
-  }
-
-  if (!_msg.name().empty() && _msg.type() != msgs::Entity::NONE)
-  {
-    Entity entity{kNullEntity};
-    if (_msg.type() == msgs::Entity::MODEL)
-    {
-      entity = _ecm.EntityByComponents(components::Model(),
-        components::Name(_msg.name()));
-    }
-    else if (_msg.type() == msgs::Entity::LIGHT)
-    {
-      entity = _ecm.EntityByComponents(
-        components::Name(_msg.name()));
-
-      auto lightComp = _ecm.Component<components::Light>(entity);
-      if (nullptr == lightComp)
-        entity = kNullEntity;
-    }
-    else
-    {
-      ignerr << "Failed to handle entity type [" << _msg.type() << "]"
-             << std::endl;
-    }
-    return entity;
-  }
-
-  ignerr << "Message missing either entity's ID or name + type" << std::endl;
-  return kNullEntity;
-}
-
-/// \brief Pose3d equality comparison function.
-/// \param[in] _a A pose to compare
-/// \param[in] _b Another pose to compare
-bool pose3Eql(const math::Pose3d &_a, const math::Pose3d &_b)
-{
-  return _a.Pos().Equal(_b.Pos(), 1e-6) &&
-    math::equal(_a.Rot().X(), _b.Rot().X(), 1e-6) &&
-    math::equal(_a.Rot().Y(), _b.Rot().Y(), 1e-6) &&
-    math::equal(_a.Rot().Z(), _b.Rot().Z(), 1e-6) &&
-    math::equal(_a.Rot().W(), _b.Rot().W(), 1e-6);
-}
-
 /// \brief This class is passed to every command and contains interfaces that
 /// can be shared among all commands. For example, all create and remove
 /// commands can use the `creator` object.
@@ -230,7 +181,50 @@ class LightCommand : public UserCommandBase
   public: std::function<bool(const msgs::Light &, const msgs::Light &)>
           lightEql { [](const msgs::Light &_a, const msgs::Light &_b)
             {
+              // todo(ahcorde) Use the field is_light_off in light.proto from
+              // Garden on.
+              auto getVisualizeVisual = [](const msgs::Light &_light) -> bool
+              {
+                bool visualizeVisual = true;
+                for (int i = 0; i < _light.header().data_size(); ++i)
+                {
+                  for (int j = 0;
+                      j < _light.header().data(i).value_size(); ++j)
+                  {
+                    if (_light.header().data(i).key() ==
+                        "visualizeVisual")
+                    {
+                      visualizeVisual = ignition::math::parseInt(
+                        _light.header().data(i).value(0));
+                    }
+                  }
+                }
+                return visualizeVisual;
+              };
+
+              // todo(ahcorde) Use the field is_light_off in light.proto from
+              // Garden on.
+              auto getIsLightOn = [](const msgs::Light &_light) -> bool
+              {
+                bool isLightOn = true;
+                for (int i = 0; i < _light.header().data_size(); ++i)
+                {
+                  for (int j = 0;
+                      j < _light.header().data(i).value_size(); ++j)
+                  {
+                    if (_light.header().data(i).key() ==
+                        "isLightOn")
+                    {
+                      isLightOn = ignition::math::parseInt(
+                        _light.header().data(i).value(0));
+                    }
+                  }
+                }
+                return isLightOn;
+               };
              return
+                getVisualizeVisual(_a) == getVisualizeVisual(_b) &&
+                getIsLightOn(_a) == getIsLightOn(_b) &&
                 _a.type() == _b.type() &&
                 _a.name() == _b.name() &&
                 math::equal(
@@ -295,6 +289,19 @@ class PoseCommand : public UserCommandBase
   public: bool Execute() final;
 };
 
+/// \brief Command to update an entity's pose transform.
+class PoseVectorCommand : public UserCommandBase
+{
+  /// \brief Constructor
+  /// \param[in] _msg pose_v message.
+  /// \param[in] _iface Pointer to user commands interface.
+  public: PoseVectorCommand(msgs::Pose_V *_msg,
+      std::shared_ptr<UserCommandsInterface> &_iface);
+
+  // Documentation inherited
+  public: bool Execute() final;
+};
+
 /// \brief Command to modify the physics parameters of a simulation.
 class PhysicsCommand : public UserCommandBase
 {
@@ -404,6 +411,48 @@ class VisualCommand : public UserCommandBase
                   aMaterial.emissive().a(), bMaterial.emissive().a(), 1e-6f);
             }};
 };
+
+/// \brief Command to modify a wheel entity from simulation.
+class WheelSlipCommand : public UserCommandBase
+{
+  /// \brief Constructor
+  /// \param[in] _msg Message containing the wheel slip parameters.
+  /// \param[in] _iface Pointer to user commands interface.
+  public: WheelSlipCommand(msgs::WheelSlipParametersCmd *_msg,
+      std::shared_ptr<UserCommandsInterface> &_iface);
+
+  // Documentation inherited
+  public: bool Execute() final;
+
+  /// \brief WheelSlip equality comparision function
+  public: std::function<bool(
+    const msgs::WheelSlipParametersCmd &, const msgs::WheelSlipParametersCmd &)>
+          wheelSlipEql {
+            [](
+              const msgs::WheelSlipParametersCmd &_a,
+              const msgs::WheelSlipParametersCmd &_b)
+            {
+              return
+                (
+                  (
+                    _a.entity().id() != kNullEntity &&
+                    _a.entity().id() == _b.entity().id()
+                  ) ||
+                  (
+                    _a.entity().name() == _b.entity().name() &&
+                    _a.entity().type() == _b.entity().type()
+                  )
+                ) &&
+                math::equal(
+                  _a.slip_compliance_lateral(),
+                  _b.slip_compliance_lateral(),
+                  1e-6) &&
+                math::equal(
+                  _a.slip_compliance_longitudinal(),
+                  _b.slip_compliance_longitudinal(),
+                  1e-6);
+            }};
+};
 }
 }
 }
@@ -443,6 +492,10 @@ class ignition::gazebo::systems::UserCommandsPrivate
   /// \return True if successful.
   public: bool LightService(const msgs::Light &_req, msgs::Boolean &_res);
 
+  /// \brief Callback for light subscription
+  /// \param[in] _msg Light message
+  public: void OnCmdLight(const msgs::Light &_msg);
+
   /// \brief Callback for pose service
   /// \param[in] _req Request containing pose update of an entity.
   /// \param[out] _res True if message successfully received and queued.
@@ -450,6 +503,13 @@ class ignition::gazebo::systems::UserCommandsPrivate
   /// \return True if successful.
   public: bool PoseService(const msgs::Pose &_req, msgs::Boolean &_res);
 
+  /// \brief Callback for pose_v service
+  /// \param[in] _req Request containing pose update of several entities.
+  /// \param[out] _res True if message successfully received and queued.
+  /// It does not mean that the entity will be successfully moved.
+  /// \return True if successful.
+  public: bool PoseVectorService(const msgs::Pose_V &_req, msgs::Boolean &_res);
+
   /// \brief Callback for physics service
   /// \param[in] _req Request containing updates to the physics parameters.
   /// \param[out] _res True if message successfully received and queued.
@@ -488,6 +548,16 @@ class ignition::gazebo::systems::UserCommandsPrivate
   /// \return True if successful.
   public: bool VisualService(const msgs::Visual &_req, msgs::Boolean &_res);
 
+  /// \brief Callback for wheel slip service
+  /// \param[in] _req Request containing wheel slip parameter updates of an
+  ///  entity.
+  /// \param[out] _res True if message sucessfully received and queued.
+  /// It does not mean that the wheel slip parameters will be successfully
+  /// updated.
+  /// \return True if successful.
+  public: bool WheelSlipService(
+    const msgs::WheelSlipParametersCmd &_req, msgs::Boolean &_res);
+
   /// \brief Queue of commands pending execution.
   public: std::vector<std::unique_ptr<UserCommandBase>> pendingCmds;
 
@@ -501,6 +571,26 @@ class ignition::gazebo::systems::UserCommandsPrivate
   public: std::mutex pendingMutex;
 };
 
+/// \brief Pose3d equality comparison function.
+/// \param[in] _a A pose to compare
+/// \param[in] _b Another pose to compare
+bool pose3Eql(const math::Pose3d &_a, const math::Pose3d &_b)
+{
+  return _a.Pos().Equal(_b.Pos(), 1e-6) &&
+    math::equal(_a.Rot().X(), _b.Rot().X(), 1e-6) &&
+    math::equal(_a.Rot().Y(), _b.Rot().Y(), 1e-6) &&
+    math::equal(_a.Rot().Z(), _b.Rot().Z(), 1e-6) &&
+    math::equal(_a.Rot().W(), _b.Rot().W(), 1e-6);
+}
+
+/// \brief Update pose for a specific pose message
+/// \param[in] _req Message containing new pose
+/// \param[in] _iface Pointer to user commands interface.
+/// \return True if successful.
+bool updatePose(
+  const msgs::Pose &_req,
+  std::shared_ptr<UserCommandsInterface> _iface);
+
 //////////////////////////////////////////////////
 UserCommands::UserCommands() : System(),
     dataPtr(std::make_unique<UserCommandsPrivate>())
@@ -594,6 +684,14 @@ void UserCommands::Configure(const Entity &_entity,
 
   ignmsg << "Pose service on [" << poseService << "]" << std::endl;
 
+  // Pose vector service
+  std::string poseVectorService{
+    "/world/" + worldName + "/set_pose_vector"};
+  this->dataPtr->node.Advertise(poseVectorService,
+      &UserCommandsPrivate::PoseVectorService, this->dataPtr.get());
+
+  ignmsg << "Pose service on [" << poseVectorService << "]" << std::endl;
+
   // Light service
   std::string lightService{"/world/" + validWorldName + "/light_config"};
   this->dataPtr->node.Advertise(lightService,
@@ -602,6 +700,10 @@ void UserCommands::Configure(const Entity &_entity,
   ignmsg << "Light configuration service on [" << lightService << "]"
     << std::endl;
 
+  std::string lightTopic{"/world/" + validWorldName + "/light_config"};
+  this->dataPtr->node.Subscribe(lightTopic, &UserCommandsPrivate::OnCmdLight,
+                                this->dataPtr.get());
+
   // Physics service
   std::string physicsService{"/world/" + validWorldName + "/set_physics"};
   this->dataPtr->node.Advertise(physicsService,
@@ -643,6 +745,14 @@ void UserCommands::Configure(const Entity &_entity,
       &UserCommandsPrivate::VisualService, this->dataPtr.get());
 
   ignmsg << "Material service on [" << visualService << "]" << std::endl;
+
+  // Wheel slip service
+  std::string wheelSlipService
+      {"/world/" + validWorldName + "/wheel_slip"};
+  this->dataPtr->node.Advertise(wheelSlipService,
+      &UserCommandsPrivate::WheelSlipService, this->dataPtr.get());
+
+  ignmsg << "Material service on [" << wheelSlipService << "]" << std::endl;
 }
 
 //////////////////////////////////////////////////
@@ -754,6 +864,21 @@ bool UserCommandsPrivate::LightService(const msgs::Light &_req,
   return true;
 }
 
+//////////////////////////////////////////////////
+void UserCommandsPrivate::OnCmdLight(const msgs::Light &_msg)
+{
+  auto msg = _msg.New();
+  msg->CopyFrom(_msg);
+  auto cmd = std::make_unique<LightCommand>(msg, this->iface);
+
+  // Push to pending
+  {
+    std::lock_guard<std::mutex> lock(this->pendingMutex);
+    this->pendingCmds.push_back(std::move(cmd));
+  }
+}
+
+
 //////////////////////////////////////////////////
 bool UserCommandsPrivate::PoseService(const msgs::Pose &_req,
     msgs::Boolean &_res)
@@ -773,6 +898,25 @@ bool UserCommandsPrivate::PoseService(const msgs::Pose &_req,
   return true;
 }
 
+//////////////////////////////////////////////////
+bool UserCommandsPrivate::PoseVectorService(const msgs::Pose_V &_req,
+    msgs::Boolean &_res)
+{
+  // Create command and push it to queue
+  auto msg = _req.New();
+  msg->CopyFrom(_req);
+  auto cmd = std::make_unique<PoseVectorCommand>(msg, this->iface);
+
+  // Push to pending
+  {
+    std::lock_guard<std::mutex> lock(this->pendingMutex);
+    this->pendingCmds.push_back(std::move(cmd));
+  }
+
+  _res.set_data(true);
+  return true;
+}
+
 //////////////////////////////////////////////////
 bool UserCommandsPrivate::EnableCollisionService(const msgs::Entity &_req,
     msgs::Boolean &_res)
@@ -847,6 +991,25 @@ bool UserCommandsPrivate::VisualService(const msgs::Visual &_req,
   return true;
 }
 
+//////////////////////////////////////////////////
+bool UserCommandsPrivate::WheelSlipService(
+    const msgs::WheelSlipParametersCmd &_req,
+    msgs::Boolean &_res)
+{
+  // Create command and push it to queue
+  auto msg = _req.New();
+  msg->CopyFrom(_req);
+  auto cmd = std::make_unique<WheelSlipCommand>(msg, this->iface);
+  // Push to pending
+  {
+    std::lock_guard<std::mutex> lock(this->pendingMutex);
+    this->pendingCmds.push_back(std::move(cmd));
+  }
+
+  _res.set_data(true);
+  return true;
+}
+
 //////////////////////////////////////////////////
 bool UserCommandsPrivate::SphericalCoordinatesService(
     const msgs::SphericalCoordinates &_req, msgs::Boolean &_res)
@@ -1089,13 +1252,14 @@ bool CreateCommand::Execute()
   this->iface->creator->SetParent(entity, this->iface->worldEntity);
 
   // Pose
+  std::optional<math::Pose3d> createPose;
   if (createMsg->has_pose())
   {
-    auto poseComp = this->iface->ecm->Component<components::Pose>(entity);
-    *poseComp = components::Pose(msgs::Convert(createMsg->pose()));
+    createPose = msgs::Convert(createMsg->pose());
   }
+
   // Spherical coordinates
-  else if (createMsg->has_spherical_coordinates())
+  if (createMsg->has_spherical_coordinates())
   {
     auto scComp = this->iface->ecm->Component<components::SphericalCoordinates>(
         this->iface->worldEntity);
@@ -1118,12 +1282,24 @@ bool CreateCommand::Execute()
           math::SphericalCoordinates::SPHERICAL,
           math::SphericalCoordinates::LOCAL2);
 
-      auto poseComp = this->iface->ecm->Component<components::Pose>(entity);
-      *poseComp = components::Pose({pos.X(), pos.Y(), pos.Z(), 0, 0,
-          IGN_DTOR(createMsg->spherical_coordinates().heading_deg())});
+      // Override pos and add to yaw
+      if (!createPose.has_value())
+        createPose = math::Pose3d::Zero;
+      createPose.value().SetX(pos.X());
+      createPose.value().SetY(pos.Y());
+      createPose.value().SetZ(pos.Z());
+      createPose.value().Rot() = math::Quaterniond(0, 0,
+          IGN_DTOR(createMsg->spherical_coordinates().heading_deg())) *
+          createPose.value().Rot();
     }
   }
 
+  if (createPose.has_value())
+  {
+    auto poseComp = this->iface->ecm->Component<components::Pose>(entity);
+    *poseComp = components::Pose(createPose.value());
+  }
+
   igndbg << "Created entity [" << entity << "] named [" << desiredName << "]"
          << std::endl;
 
@@ -1147,7 +1323,7 @@ bool RemoveCommand::Execute()
     return false;
   }
 
-  auto entity = topLevelEntityFromMessage(*this->iface->ecm, *removeMsg);
+  auto entity = entityFromMsg(*this->iface->ecm, *removeMsg);
   if (entity == kNullEntity)
   {
     ignerr << "Entity named [" << removeMsg->name() << "] of type ["
@@ -1265,6 +1441,51 @@ bool LightCommand::Execute()
   return true;
 }
 
+//////////////////////////////////////////////////
+bool updatePose(
+  const msgs::Pose &_poseMsg,
+  std::shared_ptr<UserCommandsInterface> _iface)
+{
+  // Check the name of the entity being spawned
+  std::string entityName = _poseMsg.name();
+  Entity entity = kNullEntity;
+  // TODO(anyone) Update pose message to use Entity, with default ID null
+  if (_poseMsg.id() != kNullEntity && _poseMsg.id() != 0)
+  {
+    entity = _poseMsg.id();
+  }
+  else if (!entityName.empty())
+  {
+    entity = _iface->ecm->EntityByComponents(components::Name(entityName),
+      components::ParentEntity(_iface->worldEntity));
+  }
+
+  if (!_iface->ecm->HasEntity(entity))
+  {
+    ignerr << "Unable to update the pose for entity id:[" << _poseMsg.id()
+           << "], name[" << entityName << "]" << std::endl;
+    return false;
+  }
+
+  auto poseCmdComp =
+    _iface->ecm->Component<components::WorldPoseCmd>(entity);
+  if (!poseCmdComp)
+  {
+    _iface->ecm->CreateComponent(
+        entity, components::WorldPoseCmd(msgs::Convert(_poseMsg)));
+  }
+  else
+  {
+    /// \todo(anyone) Moving an object is not captured in a log file.
+    auto state = poseCmdComp->SetData(msgs::Convert(_poseMsg), pose3Eql) ?
+        ComponentState::OneTimeChange :
+        ComponentState::NoChange;
+    _iface->ecm->SetChanged(entity, components::WorldPoseCmd::typeId,
+        state);
+  }
+  return true;
+}
+
 //////////////////////////////////////////////////
 PoseCommand::PoseCommand(msgs::Pose *_msg,
     std::shared_ptr<UserCommandsInterface> &_iface)
@@ -1282,42 +1503,32 @@ bool PoseCommand::Execute()
     return false;
   }
 
-  // Check the name of the entity being spawned
-  std::string entityName = poseMsg->name();
-  Entity entity = kNullEntity;
-  // TODO(anyone) Update pose message to use Entity, with default ID null
-  if (poseMsg->id() != kNullEntity && poseMsg->id() != 0)
-  {
-    entity = poseMsg->id();
-  }
-  else if (!entityName.empty())
-  {
-    entity = this->iface->ecm->EntityByComponents(components::Name(entityName),
-      components::ParentEntity(this->iface->worldEntity));
-  }
+  return updatePose(*poseMsg, this->iface);
+}
 
-  if (!this->iface->ecm->HasEntity(entity))
+//////////////////////////////////////////////////
+PoseVectorCommand::PoseVectorCommand(msgs::Pose_V *_msg,
+    std::shared_ptr<UserCommandsInterface> &_iface)
+    : UserCommandBase(_msg, _iface)
+{
+}
+
+//////////////////////////////////////////////////
+bool PoseVectorCommand::Execute()
+{
+  auto poseVectorMsg = dynamic_cast<const msgs::Pose_V *>(this->msg);
+  if (nullptr == poseVectorMsg)
   {
-    ignerr << "Unable to update the pose for entity id:[" << poseMsg->id()
-           << "], name[" << entityName << "]" << std::endl;
+    ignerr << "Internal error, null create message" << std::endl;
     return false;
   }
 
-  auto poseCmdComp =
-    this->iface->ecm->Component<components::WorldPoseCmd>(entity);
-  if (!poseCmdComp)
+  for (int i = 0; i < poseVectorMsg->pose_size(); i++)
   {
-    this->iface->ecm->CreateComponent(
-        entity, components::WorldPoseCmd(msgs::Convert(*poseMsg)));
-  }
-  else
-  {
-    /// \todo(anyone) Moving an object is not captured in a log file.
-    auto state = poseCmdComp->SetData(msgs::Convert(*poseMsg), pose3Eql) ?
-        ComponentState::OneTimeChange :
-        ComponentState::NoChange;
-    this->iface->ecm->SetChanged(entity, components::WorldPoseCmd::typeId,
-        state);
+    if (!updatePose(poseVectorMsg->pose(i), this->iface))
+    {
+      return false;
+    }
   }
 
   return true;
@@ -1386,7 +1597,7 @@ bool SphericalCoordinatesCommand::Execute()
   }
 
   // Entity
-  auto entity = topLevelEntityFromMessage(*this->iface->ecm,
+  auto entity = entityFromMsg(*this->iface->ecm,
       sphericalCoordinatesMsg->entity());
 
   if (!this->iface->ecm->HasEntity(entity))
@@ -1556,13 +1767,34 @@ bool VisualCommand::Execute()
     return false;
   }
 
-  if (visualMsg->id() == kNullEntity)
+  Entity visualEntity = kNullEntity;
+  if (visualMsg->id() != kNullEntity)
+  {
+    visualEntity = visualMsg->id();
+  }
+  else if (!visualMsg->name().empty() && !visualMsg->parent_name().empty())
+  {
+    Entity parentEntity =
+      this->iface->ecm->EntityByComponents(
+        components::Name(visualMsg->parent_name()));
+
+    auto entities =
+      this->iface->ecm->ChildrenByComponents(parentEntity,
+        components::Name(visualMsg->name()));
+
+    // When size > 1, we don't know which entity to modify
+    if (entities.size() == 1)
+    {
+      visualEntity = entities[0];
+    }
+  }
+
+  if (visualEntity == kNullEntity)
   {
     ignerr << "Failed to find visual entity" << std::endl;
     return false;
   }
 
-  Entity visualEntity = visualMsg->id();
   auto visualCmdComp =
       this->iface->ecm->Component<components::VisualCmd>(visualEntity);
   if (!visualCmdComp)
@@ -1580,6 +1812,67 @@ bool VisualCommand::Execute()
   return true;
 }
 
+//////////////////////////////////////////////////
+WheelSlipCommand::WheelSlipCommand(msgs::WheelSlipParametersCmd *_msg,
+    std::shared_ptr<UserCommandsInterface> &_iface)
+    : UserCommandBase(_msg, _iface)
+{
+}
+
+//////////////////////////////////////////////////
+bool WheelSlipCommand::Execute()
+{
+  auto wheelSlipMsg = dynamic_cast<const msgs::WheelSlipParametersCmd *>(
+      this->msg);
+  if (nullptr == wheelSlipMsg)
+  {
+    ignerr << "Internal error, null wheel slip message" << std::endl;
+    return false;
+  }
+  const auto & ecm = *this->iface->ecm;
+  Entity entity = entityFromMsg(ecm, wheelSlipMsg->entity());
+  if (kNullEntity == entity)
+  {
+    return false;
+  }
+
+  auto doForEachLink = [this, wheelSlipMsg](Entity linkEntity) {
+    auto wheelSlipCmdComp =
+      this->iface->ecm->Component<components::WheelSlipCmd>(linkEntity);
+    if (!wheelSlipCmdComp)
+    {
+      this->iface->ecm->CreateComponent(
+          linkEntity, components::WheelSlipCmd(*wheelSlipMsg));
+    }
+    else
+    {
+      auto state = wheelSlipCmdComp->SetData(
+        *wheelSlipMsg, this->wheelSlipEql) ? ComponentState::OneTimeChange
+        : ComponentState::NoChange;
+      this->iface->ecm->SetChanged(
+          linkEntity, components::WheelSlipCmd::typeId, state);
+    }
+  };
+  const components::BaseComponent * component =
+    ecm.Component<components::Link>(entity);
+
+  if (nullptr != component) {
+    doForEachLink(entity);
+    return true;
+  }
+  component = ecm.Component<components::Model>(entity);
+  if (nullptr != component) {
+    Model model{entity};
+    for (const auto & linkEntity : model.Links(*this->iface->ecm)) {
+      doForEachLink(linkEntity);
+    }
+    return true;
+  }
+  ignerr << "Found entity with scoped name [" << wheelSlipMsg->entity().name()
+          << "], is neither a model or a link." << std::endl;
+  return false;
+}
+
 IGNITION_ADD_PLUGIN(UserCommands, System,
   UserCommands::ISystemConfigure,
   UserCommands::ISystemPreUpdate
