@@ -22,6 +22,7 @@
 #include <ignition/common/Console.hh>
 #include <ignition/common/Util.hh>
 #include <ignition/transport/Node.hh>
+#include <ignition/utilities/ExtraTestMacros.hh>
 
 #include "ignition/gazebo/components/Joint.hh"
 #include "ignition/gazebo/components/JointPosition.hh"
@@ -47,7 +48,9 @@ class JointPositionControllerTestFixture
 
 /////////////////////////////////////////////////
 // Tests that the JointPositionController accepts joint position commands
-TEST_F(JointPositionControllerTestFixture, JointPositionForceCommand)
+// See https://github.com/ignitionrobotics/ign-gazebo/issues/1175
+TEST_F(JointPositionControllerTestFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(JointPositionForceCommand))
 {
   using namespace std::chrono_literals;
 
@@ -123,7 +126,8 @@ TEST_F(JointPositionControllerTestFixture, JointPositionForceCommand)
 
 /////////////////////////////////////////////////
 // Tests that the JointPositionController accepts joint position commands
-TEST_F(JointPositionControllerTestFixture, JointPositonVelocityCommand)
+TEST_F(JointPositionControllerTestFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(JointPositonVelocityCommand))
 {
   using namespace std::chrono_literals;
 
@@ -174,10 +178,17 @@ TEST_F(JointPositionControllerTestFixture, JointPositonVelocityCommand)
 
   server.AddSystem(testSystem.systemPtr);
 
-  const std::size_t initIters = 10;
+  // joint pos starts at 0
+  const std::size_t initIters = 1;
   server.Run(true, initIters, false);
   EXPECT_NEAR(0, currentPosition.at(0), TOL);
 
+  // joint moves to initial_position at -2.0
+  const std::size_t initPosIters = 1000;
+  server.Run(true, initPosIters, false);
+  double expectedInitialPosition = -2.0;
+  EXPECT_NEAR(expectedInitialPosition, currentPosition.at(0), TOL);
+
   // Publish command and check that the joint position is set
   transport::Node node;
   auto pub = node.Advertise<msgs::Double>(
