@@ -20,13 +20,17 @@
 #include <ignition/common/Console.hh>
 #include <ignition/common/Util.hh>
 
+#include <ignition/gazebo/components/AngularAcceleration.hh>
 #include <ignition/gazebo/components/AngularVelocity.hh>
+#include <ignition/gazebo/components/AngularVelocityCmd.hh>
 #include <ignition/gazebo/components/CanonicalLink.hh>
 #include <ignition/gazebo/components/Collision.hh>
+#include <ignition/gazebo/components/ExternalWorldWrenchCmd.hh>
 #include <ignition/gazebo/components/Inertial.hh>
 #include <ignition/gazebo/components/Joint.hh>
 #include <ignition/gazebo/components/LinearAcceleration.hh>
 #include <ignition/gazebo/components/LinearVelocity.hh>
+#include <ignition/gazebo/components/LinearVelocityCmd.hh>
 #include <ignition/gazebo/components/Link.hh>
 #include <ignition/gazebo/components/Model.hh>
 #include <ignition/gazebo/components/Name.hh>
@@ -314,23 +318,33 @@ TEST_F(LinkIntegrationTest, LinkAccelerations)
 
   // Before we enable acceleration, acceleration should return nullopt
   EXPECT_EQ(std::nullopt, link.WorldLinearAcceleration(ecm));
+  EXPECT_EQ(std::nullopt, link.WorldAngularAcceleration(ecm));
 
   // After enabling, they should return default values
   link.EnableAccelerationChecks(ecm);
   EXPECT_EQ(math::Vector3d::Zero, link.WorldLinearAcceleration(ecm));
+  EXPECT_EQ(math::Vector3d::Zero, link.WorldAngularAcceleration(ecm));
   EXPECT_NE(nullptr, ecm.Component<components::WorldLinearAcceleration>(eLink));
+  EXPECT_NE(nullptr,
+      ecm.Component<components::WorldAngularAcceleration>(eLink));
 
   // After setting acceleration, we get the value
   math::Vector3d linAccel{1.0, 0.0, 0.0};
   ecm.SetComponentData<components::WorldLinearAcceleration>(eLink, linAccel);
-
   EXPECT_EQ(linAccel, link.WorldLinearAcceleration(ecm));
 
+  math::Vector3d angAccel{0.0, 1.0, 0.0};
+  ecm.SetComponentData<components::WorldAngularAcceleration>(eLink, angAccel);
+  EXPECT_EQ(angAccel, link.WorldAngularAcceleration(ecm));
+
   // Disabling accelerations goes back to nullopt
   link.EnableAccelerationChecks(ecm, false);
 
   EXPECT_EQ(std::nullopt, link.WorldLinearAcceleration(ecm));
+  EXPECT_EQ(std::nullopt, link.WorldAngularAcceleration(ecm));
   EXPECT_EQ(nullptr, ecm.Component<components::WorldLinearAcceleration>(eLink));
+  EXPECT_EQ(nullptr,
+      ecm.Component<components::WorldAngularAcceleration>(eLink));
 }
 
 //////////////////////////////////////////////////
@@ -455,3 +469,142 @@ TEST_F(LinkIntegrationTest, LinkKineticEnergy)
   *linearVel = components::WorldLinearVelocity({math::Vector3d(10, 0, 0)});
   EXPECT_DOUBLE_EQ(100.0, *link.WorldKineticEnergy(ecm));
 }
+
+//////////////////////////////////////////////////
+TEST_F(LinkIntegrationTest, LinkSetVelocity)
+{
+  EntityComponentManager ecm;
+  EventManager eventMgr;
+  SdfEntityCreator creator(ecm, eventMgr);
+
+  auto eLink = ecm.CreateEntity();
+  ecm.CreateComponent(eLink, components::Link());
+
+  Link link(eLink);
+  EXPECT_EQ(eLink, link.Entity());
+
+  ASSERT_TRUE(link.Valid(ecm));
+
+  // No LinearVelocityCmd should exist by default
+  EXPECT_EQ(nullptr, ecm.Component<components::LinearVelocityCmd>(eLink));
+
+  math::Vector3d linVel(1, 0, 0);
+  link.SetLinearVelocity(ecm, linVel);
+
+  // LinearVelocity cmd should exist
+  EXPECT_NE(nullptr, ecm.Component<components::LinearVelocityCmd>(eLink));
+  EXPECT_EQ(linVel,
+    ecm.Component<components::LinearVelocityCmd>(eLink)->Data());
+
+  // Make sure the linear velocity is updated
+  math::Vector3d linVel2(0, 0, 0);
+  link.SetLinearVelocity(ecm, linVel2);
+  EXPECT_EQ(linVel2,
+    ecm.Component<components::LinearVelocityCmd>(eLink)->Data());
+
+  // No AngularVelocityCmd should exist by default
+  EXPECT_EQ(nullptr, ecm.Component<components::AngularVelocityCmd>(eLink));
+
+  math::Vector3d angVel(0, 0, 1);
+  link.SetAngularVelocity(ecm, angVel);
+
+  // AngularVelocity cmd should exist
+  EXPECT_NE(nullptr, ecm.Component<components::AngularVelocityCmd>(eLink));
+  EXPECT_EQ(angVel,
+    ecm.Component<components::AngularVelocityCmd>(eLink)->Data());
+
+  // Make sure the angular velocity is updated
+  math::Vector3d angVel2(0, 0, 0);
+  link.SetAngularVelocity(ecm, angVel2);
+  EXPECT_EQ(angVel2,
+    ecm.Component<components::AngularVelocityCmd>(eLink)->Data());
+}
+
+//////////////////////////////////////////////////
+TEST_F(LinkIntegrationTest, LinkAddWorldForce)
+{
+  EntityComponentManager ecm;
+  EventManager eventMgr;
+  SdfEntityCreator creator(ecm, eventMgr);
+
+  auto eLink = ecm.CreateEntity();
+  ecm.CreateComponent(eLink, components::Link());
+
+  Link link(eLink);
+  EXPECT_EQ(eLink, link.Entity());
+
+  ASSERT_TRUE(link.Valid(ecm));
+
+  // No ExternalWorldWrenchCmd should exist by default
+  EXPECT_EQ(nullptr, ecm.Component<components::ExternalWorldWrenchCmd>(eLink));
+
+  // Add force
+  math::Vector3d force(0, 0, 1.0);
+  link.AddWorldForce(ecm, force);
+
+  // No WorldPose or Inertial component exists so command should not work
+  EXPECT_EQ(nullptr, ecm.Component<components::ExternalWorldWrenchCmd>(eLink));
+
+  // create WorldPose and Inertial component and try adding force again
+  math::Pose3d linkWorldPose;
+  linkWorldPose.Set(1.0, 0.0, 0.0, 0, 0, IGN_PI_4);
+  math::Pose3d inertiaPose;
+  inertiaPose.Set(1.0, 2.0, 3.0, 0, IGN_PI_2, 0);
+  math::Inertiald linkInertial;
+  linkInertial.SetPose(inertiaPose);
+  ecm.CreateComponent(eLink, components::WorldPose(linkWorldPose));
+  ecm.CreateComponent(eLink, components::Inertial(linkInertial));
+  link.AddWorldForce(ecm, force);
+
+  // ExternalWorldWrenchCmd component should now be created
+  auto wrenchComp = ecm.Component<components::ExternalWorldWrenchCmd>(eLink);
+  EXPECT_NE(nullptr, wrenchComp);
+
+  // verify wrench values
+  auto wrenchMsg = wrenchComp->Data();
+
+  math::Vector3 expectedTorque =
+      linkWorldPose.Rot().RotateVector(inertiaPose.Pos()).Cross(force);
+  EXPECT_EQ(force, math::Vector3d(
+      wrenchMsg.force().x(), wrenchMsg.force().y(), wrenchMsg.force().z()));
+  EXPECT_EQ(expectedTorque, math::Vector3d(
+      wrenchMsg.torque().x(), wrenchMsg.torque().y(), wrenchMsg.torque().z()));
+
+  // apply opposite force. Since the cmd is not processed yet, this should
+  // cancel out the existing wrench cmd
+  link.AddWorldForce(ecm, -force);
+  wrenchComp = ecm.Component<components::ExternalWorldWrenchCmd>(eLink);
+  EXPECT_NE(nullptr, wrenchComp);
+  wrenchMsg = wrenchComp->Data();
+
+  EXPECT_EQ(math::Vector3d::Zero, math::Vector3d(
+      wrenchMsg.force().x(), wrenchMsg.force().y(), wrenchMsg.force().z()));
+  EXPECT_EQ(math::Vector3d::Zero, math::Vector3d(
+      wrenchMsg.torque().x(), wrenchMsg.torque().y(), wrenchMsg.torque().z()));
+
+  // Add world force at an offset
+  math::Vector3d offset{0.0, 1.0, 0.0};
+  link.AddWorldForce(ecm, force, offset);
+
+  wrenchComp = ecm.Component<components::ExternalWorldWrenchCmd>(eLink);
+  EXPECT_NE(nullptr, wrenchComp);
+  wrenchMsg = wrenchComp->Data();
+
+  expectedTorque =
+      linkWorldPose.Rot().RotateVector(offset + inertiaPose.Pos()).Cross(force);
+  EXPECT_EQ(force, math::Vector3d(
+      wrenchMsg.force().x(), wrenchMsg.force().y(), wrenchMsg.force().z()));
+  EXPECT_EQ(expectedTorque, math::Vector3d(
+      wrenchMsg.torque().x(), wrenchMsg.torque().y(), wrenchMsg.torque().z()));
+
+  // apply opposite force again and verify the resulting wrench values are zero
+  link.AddWorldForce(ecm, -force, offset);
+  wrenchComp = ecm.Component<components::ExternalWorldWrenchCmd>(eLink);
+  EXPECT_NE(nullptr, wrenchComp);
+  wrenchMsg = wrenchComp->Data();
+
+  EXPECT_EQ(math::Vector3d::Zero, math::Vector3d(
+      wrenchMsg.force().x(), wrenchMsg.force().y(), wrenchMsg.force().z()));
+  EXPECT_EQ(math::Vector3d::Zero, math::Vector3d(
+      wrenchMsg.torque().x(), wrenchMsg.torque().y(), wrenchMsg.torque().z()));
+}
