@@ -37,6 +37,7 @@
 #include <ignition/transport/log/Playback.hh>
 #include <ignition/transport/log/QualifiedTime.hh>
 #include <ignition/math/Pose3.hh>
+#include <ignition/utilities/ExtraTestMacros.hh>
 
 #include <sdf/Root.hh>
 #include <sdf/World.hh>
@@ -262,7 +263,8 @@ class LogSystemTest : public InternalFixture<::testing::Test>
 };
 
 /////////////////////////////////////////////////
-TEST_F(LogSystemTest, LogPlaybackStatistics)
+// See https://github.com/ignitionrobotics/ign-gazebo/issues/1175
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LogPlaybackStatistics))
 {
   // TODO(anyone) see LogSystemTest.LogControl comment about re-recording
   auto logPath = common::joinPaths(PROJECT_SOURCE_PATH, "test", "media",
@@ -314,7 +316,7 @@ TEST_F(LogSystemTest, LogPlaybackStatistics)
 
 /////////////////////////////////////////////////
 // Logging behavior when no paths are specified
-TEST_F(LogSystemTest, LogDefaults)
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LogDefaults))
 {
   // Create temp directory to store log
   this->CreateLogsDir();
@@ -332,8 +334,7 @@ TEST_F(LogSystemTest, LogDefaults)
 
   // Test case 1:
   // No path specified on command line. This does not go through
-  // ign.cc, so ignLogDirectory() is not initialized (empty string). Recording
-  // should not take place.
+  // ign.cc, recording should take place in the `.ignition` directory
   {
     // Load SDF
     sdf::Root recordSdfRoot;
@@ -353,8 +354,12 @@ TEST_F(LogSystemTest, LogDefaults)
     recordServer.Run(true, 200, false);
   }
 
-  // Check ignLogDirectory is empty
-  EXPECT_TRUE(ignLogDirectory().empty());
+  // We should expect to see "auto_default.log"  and "state.tlog"
+  EXPECT_FALSE(ignLogDirectory().empty());
+  EXPECT_TRUE(common::exists(
+        common::joinPaths(ignLogDirectory(), "auto_default.log")));
+  EXPECT_TRUE(common::exists(
+        common::joinPaths(ignLogDirectory(), "state.tlog")));
 
   // Remove artifacts. Recreate new directory
   this->RemoveLogsDir();
@@ -418,7 +423,7 @@ TEST_F(LogSystemTest, LogDefaults)
 /////////////////////////////////////////////////
 // Logging behavior when a path is specified either via the C++ API, SDF, or
 // the command line.
-TEST_F(LogSystemTest, LogPaths)
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LogPaths))
 {
   // Create temp directory to store log
   this->CreateLogsDir();
@@ -687,7 +692,7 @@ TEST_F(LogSystemTest, LogPaths)
 }
 
 /////////////////////////////////////////////////
-TEST_F(LogSystemTest, RecordAndPlayback)
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(RecordAndPlayback))
 {
   // Create temp directory to store log
   this->CreateLogsDir();
@@ -837,7 +842,7 @@ TEST_F(LogSystemTest, RecordAndPlayback)
 }
 
 /////////////////////////////////////////////////
-TEST_F(LogSystemTest, LogControl)
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LogControl))
 {
   // TODO(anyone) when re-recording state.tlog file, do not run
   // `ign gazebo --record rolling_shapes.sdf` with `-r` flag and pause sim
@@ -957,7 +962,7 @@ TEST_F(LogSystemTest, LogControl)
 }
 
 /////////////////////////////////////////////////
-TEST_F(LogSystemTest, LogOverwrite)
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LogOverwrite))
 {
   // Create temp directory to store log
   this->CreateLogsDir();
@@ -1106,7 +1111,7 @@ TEST_F(LogSystemTest, LogOverwrite)
 }
 
 /////////////////////////////////////////////////
-TEST_F(LogSystemTest, LogControlLevels)
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LogControlLevels))
 {
   auto logPath = common::joinPaths(PROJECT_SOURCE_PATH, "test", "media",
       "levels_log");
@@ -1245,7 +1250,7 @@ TEST_F(LogSystemTest, LogControlLevels)
 }
 
 /////////////////////////////////////////////////
-TEST_F(LogSystemTest, LogCompress)
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LogCompress))
 {
   // Create temp directory to store log
   this->CreateLogsDir();
@@ -1351,7 +1356,7 @@ TEST_F(LogSystemTest, LogCompress)
 }
 
 /////////////////////////////////////////////////
-TEST_F(LogSystemTest, LogCompressOverwrite)
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LogCompressOverwrite))
 {
   // Create temp directory to store log
   this->CreateLogsDir();
@@ -1397,7 +1402,7 @@ TEST_F(LogSystemTest, LogCompressOverwrite)
 }
 
 /////////////////////////////////////////////////
-TEST_F(LogSystemTest, LogCompressCmdLine)
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LogCompressCmdLine))
 {
 #ifndef __APPLE__
   // Create temp directory to store log
@@ -1477,7 +1482,7 @@ TEST_F(LogSystemTest, LogCompressCmdLine)
 }
 
 /////////////////////////////////////////////////
-TEST_F(LogSystemTest, LogResources)
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LogResources))
 {
   // Create temp directory to store log
   this->CreateLogsDir();
@@ -1564,7 +1569,7 @@ TEST_F(LogSystemTest, LogResources)
 }
 
 /////////////////////////////////////////////////
-TEST_F(LogSystemTest, LogTopics)
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(LogTopics))
 {
   // Create temp directory to store log
   this->CreateLogsDir();
@@ -1638,3 +1643,84 @@ TEST_F(LogSystemTest, LogTopics)
   this->CreateLogsDir();
 #endif
 }
+
+/////////////////////////////////////////////////
+TEST_F(LogSystemTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(RecordPeriod))
+{
+  // Create temp directory to store log
+  this->CreateLogsDir();
+
+  // test world
+  const auto recordSdfPath = common::joinPaths(
+    std::string(PROJECT_SOURCE_PATH), "test", "worlds",
+    "log_record_resources.sdf");
+
+  // Change environment variable so that downloaded fuel files aren't written
+  // to $HOME
+  std::string homeOrig;
+  common::env(IGN_HOMEDIR, homeOrig);
+  std::string homeFake = common::joinPaths(this->logsDir, "default");
+  EXPECT_TRUE(ignition::common::setenv(IGN_HOMEDIR, homeFake.c_str()));
+
+  const std::string recordPath = this->logDir;
+  std::string statePath = common::joinPaths(recordPath, "state.tlog");
+
+  int numIterations = 100;
+#ifndef __APPLE__
+  // Log from command line
+  {
+    // Command line triggers ign.cc, which handles initializing ignLogDirectory
+    std::string cmd = kIgnCommand + " -r -v 4 --iterations "
+      + std::to_string(numIterations) + " "
+      + "--record-period 0.002 "
+      + "--record-path " + recordPath + " " + recordSdfPath;
+    std::cout << "Running command [" << cmd << "]" << std::endl;
+
+    // Run
+    std::string output = customExecStr(cmd);
+    std::cout << output << std::endl;
+  }
+
+  std::string consolePath = common::joinPaths(recordPath, "server_console.log");
+  EXPECT_TRUE(common::exists(consolePath)) << consolePath;
+  EXPECT_TRUE(common::exists(statePath)) << statePath;
+
+  // Recorded models should exist
+  EXPECT_GT(entryCount(recordPath), 1);
+
+  // Verify file is created
+  auto logFile = common::joinPaths(recordPath, "state.tlog");
+  EXPECT_TRUE(common::exists(logFile));
+
+  // Load the state log file into a player.
+  transport::log::Playback player(statePath);
+  const int64_t addTopicResult = player.AddTopic(std::regex(".*"));
+
+  // There should be 2 topics (sdf, & state)
+  EXPECT_EQ(2, addTopicResult);
+
+  int msgCount = 0;
+  std::function<void(const msgs::SerializedStateMap &)> stateCb =
+      [&](const msgs::SerializedStateMap &) -> void
+  {
+    msgCount++;
+  };
+
+  // Subscribe to the state topic
+  transport::Node node;
+  node.Subscribe("/world/default/changed_state", stateCb);
+
+  // Begin playback
+  transport::log::PlaybackHandlePtr handle =
+    player.Start(std::chrono::seconds(5), false);
+  handle->WaitUntilFinished();
+
+  // There were 100 iterations of simulation, and we were recording at 2ms
+  // so there should be 50 state messages.
+  EXPECT_EQ(50, msgCount);
+
+  // Remove artifacts. Recreate new directory
+  this->RemoveLogsDir();
+  this->CreateLogsDir();
+#endif
+}
