@@ -19,12 +19,16 @@
 #include <chrono>
 #include <condition_variable>
 
+#include <ignition/utilities/ExtraTestMacros.hh>
+
 #include "ignition/msgs/world_control.pb.h"
 #include "ignition/msgs/world_stats.pb.h"
 #include "ignition/transport/Node.hh"
 #include "ignition/gazebo/Server.hh"
 #include "ignition/gazebo/test_config.hh"  // NOLINT(build/include)
 
+#include "../helpers/EnvTestFixture.hh"
+
 using namespace ignition;
 using namespace gazebo;
 using namespace std::chrono_literals;
@@ -59,17 +63,13 @@ uint64_t testPaused(bool _paused)
 }
 
 /////////////////////////////////////////////////
-class NetworkHandshake : public ::testing::Test
+class NetworkHandshake : public InternalFixture<::testing::Test>
 {
-  // Documentation inherited
-  protected: void SetUp() override
-  {
-    common::Console::SetVerbosity(4);
-  }
 };
 
 /////////////////////////////////////////////////
-TEST_F(NetworkHandshake, Handshake)
+// See https://github.com/ignitionrobotics/ign-gazebo/issues/1175
+TEST_F(NetworkHandshake, IGN_UTILS_TEST_DISABLED_ON_WIN32(Handshake))
 {
   ServerConfig serverConfig;
   serverConfig.SetSdfString(TestWorldSansPhysics::World());
@@ -125,7 +125,8 @@ TEST_F(NetworkHandshake, Handshake)
 }
 
 /////////////////////////////////////////////////
-TEST_F(NetworkHandshake, Updates)
+// See: https://github.com/gazebosim/gz-sim/issues/630
+TEST_F(NetworkHandshake, IGN_UTILS_TEST_ENABLED_ONLY_ON_LINUX(Updates))
 {
   auto pluginElem = std::make_shared<sdf::Element>();
   pluginElem->SetName("plugin");
@@ -136,10 +137,11 @@ TEST_F(NetworkHandshake, Updates)
   ServerConfig::PluginInfo primaryPluginInfo;
   primaryPluginInfo.SetEntityName("default");
   primaryPluginInfo.SetEntityType("world");
-  primaryPluginInfo.SetFilename(
-      "libignition-gazebo-scene-broadcaster-system.so");
-  primaryPluginInfo.SetName("ignition::gazebo::systems::SceneBroadcaster");
-  primaryPluginInfo.SetSdf(pluginElem);
+  sdf::Plugin plugin;
+  plugin.SetFilename("libignition-gazebo-scene-broadcaster-system.so");
+  plugin.SetName("ignition::gazebo::systems::SceneBroadcaster");
+  plugin.InsertContent(pluginElem);
+  primaryPluginInfo.SetPlugin(plugin);
 
   ServerConfig configPrimary;
   configPrimary.SetNetworkRole("primary");
@@ -158,9 +160,11 @@ TEST_F(NetworkHandshake, Updates)
   ServerConfig::PluginInfo secondaryPluginInfo;
   secondaryPluginInfo.SetEntityName("default");
   secondaryPluginInfo.SetEntityType("world");
-  secondaryPluginInfo.SetFilename("libignition-gazebo-physics-system.so");
-  secondaryPluginInfo.SetName("ignition::gazebo::systems::Physics");
-  secondaryPluginInfo.SetSdf(pluginElem);
+  sdf::Plugin secondPlugin;
+  secondPlugin.SetFilename("libignition-gazebo-physics-system.so");
+  secondPlugin.SetName("ignition::gazebo::systems::Physics");
+  secondPlugin.InsertContent(pluginElem);
+  secondaryPluginInfo.SetPlugin(secondPlugin);
 
   ServerConfig configSecondary;
   configSecondary.SetNetworkRole("secondary");
