@@ -27,6 +27,7 @@
 #include <ignition/common/Console.hh>
 #include <ignition/common/Util.hh>
 #include <ignition/msgs/Utility.hh>
+#include <ignition/utilities/ExtraTestMacros.hh>
 #include <sdf/Collision.hh>
 #include <sdf/Cylinder.hh>
 #include <sdf/Geometry.hh>
@@ -117,8 +118,9 @@ TEST_F(PhysicsSystemFixture, CreatePhysicsWorld)
   // TODO(addisu) add useful EXPECT calls
 }
 
-/////////////////////////////////////////////////
-TEST_F(PhysicsSystemFixture, FallingObject)
+////////////////////////////////////////////////
+// See https://github.com/ignitionrobotics/ign-gazebo/issues/1175
+TEST_F(PhysicsSystemFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(FallingObject))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -187,7 +189,7 @@ TEST_F(PhysicsSystemFixture, FallingObject)
 // This tests whether links with fixed joints keep their relative transforms
 // after physics. For that to work properly, the canonical link implementation
 // must be correct.
-TEST_F(PhysicsSystemFixture, CanonicalLink)
+TEST_F(PhysicsSystemFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(CanonicalLink))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -258,7 +260,8 @@ TEST_F(PhysicsSystemFixture, CanonicalLink)
 
 /////////////////////////////////////////////////
 // Same as the CanonicalLink test, but with a non-default canonical link
-TEST_F(PhysicsSystemFixture, NonDefaultCanonicalLink)
+TEST_F(PhysicsSystemFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(NonDefaultCanonicalLink))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -313,7 +316,7 @@ TEST_F(PhysicsSystemFixture, NonDefaultCanonicalLink)
 
 /////////////////////////////////////////////////
 // Test physics integration with revolute joints
-TEST_F(PhysicsSystemFixture, RevoluteJoint)
+TEST_F(PhysicsSystemFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(RevoluteJoint))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -392,7 +395,7 @@ TEST_F(PhysicsSystemFixture, RevoluteJoint)
 }
 
 /////////////////////////////////////////////////
-TEST_F(PhysicsSystemFixture, CreateRuntime)
+TEST_F(PhysicsSystemFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(CreateRuntime))
 {
   ignition::gazebo::ServerConfig serverConfig;
   gazebo::Server server(serverConfig);
@@ -475,7 +478,8 @@ TEST_F(PhysicsSystemFixture, CreateRuntime)
 }
 
 /////////////////////////////////////////////////
-TEST_F(PhysicsSystemFixture, SetFrictionCoefficient)
+TEST_F(PhysicsSystemFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(SetFrictionCoefficient))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -560,7 +564,8 @@ TEST_F(PhysicsSystemFixture, SetFrictionCoefficient)
 
 /////////////////////////////////////////////////
 /// Test that joint position reported by the physics system include all axes
-TEST_F(PhysicsSystemFixture, MultiAxisJointPosition)
+TEST_F(PhysicsSystemFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(MultiAxisJointPosition))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -639,7 +644,8 @@ TEST_F(PhysicsSystemFixture, MultiAxisJointPosition)
 
 /////////////////////////////////////////////////
 /// Test joint position reset component
-TEST_F(PhysicsSystemFixture, ResetPositionComponent)
+TEST_F(PhysicsSystemFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(ResetPositionComponent))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -738,7 +744,8 @@ TEST_F(PhysicsSystemFixture, ResetPositionComponent)
 
 /////////////////////////////////////////////////
 /// Test joint veocity reset component
-TEST_F(PhysicsSystemFixture, ResetVelocityComponent)
+TEST_F(PhysicsSystemFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(ResetVelocityComponent))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -1210,7 +1217,7 @@ TEST_F(PhysicsSystemFixtureWithDart6_10, JointEffortLimitsCommandComponent)
 }
 
 /////////////////////////////////////////////////
-TEST_F(PhysicsSystemFixture, GetBoundingBox)
+TEST_F(PhysicsSystemFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(GetBoundingBox))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -1281,7 +1288,7 @@ TEST_F(PhysicsSystemFixture, GetBoundingBox)
 
 /////////////////////////////////////////////////
 // This tests whether nested models can be loaded correctly
-TEST_F(PhysicsSystemFixture, NestedModel)
+TEST_F(PhysicsSystemFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(NestedModel))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -1386,7 +1393,8 @@ TEST_F(PhysicsSystemFixture, NestedModel)
 }
 
 // This tests whether nested models can be loaded correctly
-TEST_F(PhysicsSystemFixture, IncludeNestedModelDartsim)
+TEST_F(PhysicsSystemFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(IncludeNestedModelDartsim))
 {
   std::string path = std::string(PROJECT_SOURCE_PATH) + "/test/worlds/models";
   ignition::common::setenv("IGN_GAZEBO_RESOURCE_PATH", path.c_str());
@@ -1528,7 +1536,8 @@ TEST_F(PhysicsSystemFixture, IncludeNestedModelDartsim)
 }
 
 // This tests whether nested models can be loaded correctly
-TEST_F(PhysicsSystemFixture, IncludeNestedModelTPE)
+TEST_F(PhysicsSystemFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(IncludeNestedModelTPE))
 {
   std::string path = std::string(PROJECT_SOURCE_PATH) + "/test/worlds/models";
   ignition::common::setenv("IGN_GAZEBO_RESOURCE_PATH", path.c_str());
@@ -1670,7 +1679,8 @@ TEST_F(PhysicsSystemFixture, IncludeNestedModelTPE)
 }
 
 // This tests whether the poses of nested models are updated correctly
-TEST_F(PhysicsSystemFixture, NestedModelIndividualCanonicalLinks)
+TEST_F(PhysicsSystemFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(NestedModelIndividualCanonicalLinks))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -1771,7 +1781,8 @@ TEST_F(PhysicsSystemFixture, NestedModelIndividualCanonicalLinks)
 }
 
 /////////////////////////////////////////////////
-TEST_F(PhysicsSystemFixture, DefaultPhysicsOptions)
+TEST_F(PhysicsSystemFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(DefaultPhysicsOptions))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -1857,7 +1868,8 @@ TEST_F(PhysicsSystemFixture, PhysicsOptions)
 /////////////////////////////////////////////////
 // This tests whether pose updates are correct for a model whose canonical link
 // changes, but other links do not
-TEST_F(PhysicsSystemFixture, MovingCanonicalLinkOnly)
+TEST_F(PhysicsSystemFixture,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(MovingCanonicalLinkOnly))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -1984,7 +1996,7 @@ TEST_F(PhysicsSystemFixture, MovingCanonicalLinkOnly)
 }
 
 /////////////////////////////////////////////////
-TEST_F(PhysicsSystemFixture, Heightmap)
+TEST_F(PhysicsSystemFixture, IGN_UTILS_TEST_DISABLED_ON_WIN32(Heightmap))
 {
   ignition::gazebo::ServerConfig serverConfig;
 
@@ -2065,7 +2077,8 @@ TEST_F(PhysicsSystemFixture, Heightmap)
 
 /////////////////////////////////////////////////
 // Joint force
-TEST_F(PhysicsSystemFixture, JointTransmittedWrench)
+TEST_F(PhysicsSystemFixture,
+    IGN_UTILS_TEST_DISABLED_ON_WIN32(JointTransmittedWrench))
 {
   common::Console::SetVerbosity(4);
   ignition::gazebo::ServerConfig serverConfig;
@@ -2183,3 +2196,71 @@ TEST_F(PhysicsSystemFixture, JointTransmittedWrench)
     EXPECT_NEAR(0.0, wrench.torque().z(), 1e-3);
   }
 }
+
+/////////////////////////////////////////////////
+// Test that joint velocity limit is applied
+TEST_F(PhysicsSystemFixtureWithDart6_10,
+    IGN_UTILS_TEST_DISABLED_ON_WIN32(JointVelocityLimitTest))
+{
+  // Start server
+  ServerConfig serverConfig;
+  const auto sdfFile = std::string(PROJECT_SOURCE_PATH) +
+          "/test/worlds/joint_velocity_limit.sdf";
+  serverConfig.SetSdfFile(sdfFile);
+
+  auto server = std::make_unique<Server>(serverConfig);
+  EXPECT_FALSE(server->Running());
+  EXPECT_FALSE(*server->Running(0));
+
+  server->SetUpdatePeriod(1ns);
+
+  test::Relay testSystem;
+
+  const std::size_t nIters{600};
+  testSystem.OnPreUpdate(
+      [&](const gazebo::UpdateInfo &_info, gazebo::EntityComponentManager &_ecm)
+      {
+        // Create components, if they don't exist on the first iteration
+        if (_info.iterations == 1)
+        {
+          for (const auto &e : _ecm.EntitiesByComponents(components::Joint()))
+          {
+            if (!_ecm.Component<components::JointVelocity>(e))
+            {
+              _ecm.CreateComponent(e, components::JointVelocity());
+            }
+          }
+        }
+      });
+
+  testSystem.OnPostUpdate(
+      [&](const gazebo::UpdateInfo &_info,
+          const gazebo::EntityComponentManager &_ecm)
+      {
+        // At nIters iterations, check angular velocity of each of the joints
+        if (_info.iterations == nIters)
+        {
+          int count = 0;
+          for (const auto &e : _ecm.EntitiesByComponents(components::Joint()))
+          {
+            auto *jointVel = _ecm.Component<components::JointVelocity>(e);
+            EXPECT_NE(nullptr, jointVel);
+            EXPECT_FALSE(jointVel->Data().empty());
+            if (jointVel->Data().size() > 0)
+            {
+              ++count;
+              // Joint velocity should lie between
+              // - (kVelocityLimit + kTolerance) and
+              // + (kVelocityLimit + kTolerance)
+              const double kVelocityLimit = 1.0;
+              const double kTolerance = 1e-6;
+              EXPECT_NEAR(jointVel->Data()[0], 0, kVelocityLimit + kTolerance);
+            }
+          }
+          EXPECT_EQ(count, 2);
+        }
+      });
+
+  server->AddSystem(testSystem.systemPtr);
+  server->Run(true, nIters, false);
+}
