@@ -22,6 +22,7 @@
 #include <ignition/common/Util.hh>
 #include <ignition/math/Pose3.hh>
 #include <ignition/transport/Node.hh>
+#include <ignition/utilities/ExtraTestMacros.hh>
 
 #include "ignition/gazebo/components/Name.hh"
 #include "ignition/gazebo/components/Model.hh"
@@ -98,7 +99,8 @@ std::string addDelimiter(const std::vector<std::string> &_name,
 }
 
 /////////////////////////////////////////////////
-TEST_F(PosePublisherTest, PublishCmd)
+// See https://github.com/ignitionrobotics/ign-gazebo/issues/1175
+TEST_F(PosePublisherTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(PublishCmd))
 {
   // Start server
   ServerConfig serverConfig;
@@ -320,7 +322,9 @@ TEST_F(PosePublisherTest, PublishCmd)
 }
 
 /////////////////////////////////////////////////
-TEST_F(PosePublisherTest, UpdateFrequency)
+// See: https://github.com/gazebosim/gz-sim/issues/630
+TEST_F(PosePublisherTest,
+       IGN_UTILS_TEST_ENABLED_ONLY_ON_LINUX(UpdateFrequency))
 {
   // Start server
   ServerConfig serverConfig;
@@ -385,7 +389,7 @@ TEST_F(PosePublisherTest, UpdateFrequency)
 }
 
 /////////////////////////////////////////////////
-TEST_F(PosePublisherTest, StaticPosePublisher)
+TEST_F(PosePublisherTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(StaticPosePublisher))
 {
   // Start server
   ServerConfig serverConfig;
@@ -641,7 +645,9 @@ TEST_F(PosePublisherTest, StaticPosePublisher)
 }
 
 /////////////////////////////////////////////////
-TEST_F(PosePublisherTest, StaticPoseUpdateFrequency)
+// See: https://github.com/gazebosim/gz-sim/issues/630
+TEST_F(PosePublisherTest,
+       IGN_UTILS_TEST_ENABLED_ONLY_ON_LINUX(StaticPoseUpdateFrequency))
 {
   // Start server
   ServerConfig serverConfig;
@@ -708,7 +714,8 @@ TEST_F(PosePublisherTest, StaticPoseUpdateFrequency)
 }
 
 /////////////////////////////////////////////////
-TEST_F(PosePublisherTest, NestedModelLoadPlugin)
+TEST_F(PosePublisherTest,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(NestedModelLoadPlugin))
 {
   // Start server
   ServerConfig serverConfig;
@@ -738,4 +745,19 @@ TEST_F(PosePublisherTest, NestedModelLoadPlugin)
   }
 
   EXPECT_TRUE(!poseMsgs.empty());
+
+  // only the pose of the nested model should be published and no other entity
+  std::string expectedEntityName = "model_00::model_01";
+  math::Pose3d expectedEntityPose(1, 0, 0, 0, 0, 0);
+  for (auto &msg : poseMsgs)
+  {
+    ASSERT_LT(1, msg.header().data_size());
+    ASSERT_LT(0, msg.header().data(1).value_size());
+    std::string childFrameId = msg.header().data(1).value(0);
+    EXPECT_EQ(expectedEntityName, childFrameId);
+    math::Pose3d pose;
+    auto p = msgs::Convert(poseMsgs[0]);
+    EXPECT_EQ(expectedEntityPose, p);
+  }
+
 }
