@@ -18,6 +18,7 @@
 #include <gtest/gtest.h>
 
 #include <string>
+#include <unordered_set>
 #include <vector>
 
 #include <sdf/Model.hh>
@@ -27,6 +28,11 @@
 #include <ignition/transport/Node.hh>
 #include <ignition/utilities/ExtraTestMacros.hh>
 
+#include <ignition/rendering/Camera.hh>
+#include <ignition/rendering/RenderEngine.hh>
+#include <ignition/rendering/RenderingIface.hh>
+#include <ignition/rendering/Scene.hh>
+
 #include "ignition/gazebo/EntityComponentManager.hh"
 #include "ignition/gazebo/EventManager.hh"
 #include "ignition/gazebo/SdfEntityCreator.hh"
@@ -37,8 +43,11 @@
 
 #include "ignition/gazebo/components/Model.hh"
 #include "ignition/gazebo/components/Name.hh"
+#include "ignition/gazebo/components/Sensor.hh"
 #include "ignition/gazebo/components/World.hh"
 
+#include "ignition/gazebo/rendering/Events.hh"
+
 #include "plugins/MockSystem.hh"
 #include "../helpers/EnvTestFixture.hh"
 
@@ -46,6 +55,45 @@ using namespace ignition;
 using namespace std::chrono_literals;
 namespace components = ignition::gazebo::components;
 
+std::unordered_set<gazebo::Entity> g_sensorEntityIds;
+rendering::ScenePtr g_scene;
+
+/////////////////////////////////////////////////
+void OnPostRender()
+{
+  if (!g_scene)
+  {
+    g_scene = rendering::sceneFromFirstRenderEngine();
+  }
+  ASSERT_TRUE(g_scene);
+
+  EXPECT_LT(0u, g_scene->SensorCount());
+  for (unsigned int i = 0; i < g_scene->SensorCount(); ++i)
+  {
+    auto sensor = g_scene->SensorByIndex(i);
+    ASSERT_TRUE(sensor);
+    EXPECT_TRUE(sensor->HasUserData("gazebo-entity"));
+    auto variant = sensor->UserData("gazebo-entity");
+
+    // todo(anyone) change int to uint64_t once user data supports it
+    const int *value = std::get_if<int>(&variant);
+    ASSERT_TRUE(value);
+    g_sensorEntityIds.insert(*value);
+  }
+}
+
+/////////////////////////////////////////////////
+void testSensorEntityIds(const gazebo::EntityComponentManager &_ecm,
+    const std::unordered_set<gazebo::Entity> &_ids)
+{
+  EXPECT_FALSE(_ids.empty());
+  for (const auto & id : _ids)
+  {
+    auto sensorComp = _ecm.Component<gazebo::components::Sensor>(id);
+    EXPECT_TRUE(sensorComp);
+  }
+}
+
 //////////////////////////////////////////////////
 class SensorsFixture : public InternalFixture<InternalFixture<::testing::Test>>
 {
@@ -53,9 +101,10 @@ class SensorsFixture : public InternalFixture<InternalFixture<::testing::Test>>
   {
     InternalFixture::SetUp();
 
-    auto plugin = sm.LoadPlugin("libMockSystem.so",
-                                "ignition::gazebo::MockSystem",
-                                nullptr);
+    sdf::Plugin sdfPlugin;
+    sdfPlugin.SetFilename("libMockSystem.so");
+    sdfPlugin.SetName("ignition::gazebo::MockSystem");
+    auto plugin = sm.LoadPlugin(sdfPlugin);
     EXPECT_TRUE(plugin.has_value());
     this->systemPtr = plugin.value();
     this->mockSystem = static_cast<gazebo::MockSystem *>(
@@ -87,6 +136,17 @@ void testDefaultTopics()
   std::vector<transport::MessagePublisher> publishers;
   transport::Node node;
 
+  // Sensors are created in a separate thread, so we sleep here to give them
+  // time
+  int sleep{0};
+  int maxSleep{30};
+  for (; sleep < maxSleep && !node.TopicInfo(topics.front(), publishers);
+      ++sleep)
+  {
+    std::this_thread::sleep_for(std::chrono::milliseconds(100));
+  }
+  ASSERT_LT(sleep, maxSleep);
+
   for (const std::string &topic : topics)
   {
     bool result = node.TopicInfo(topic, publishers);
@@ -116,6 +176,8 @@ TEST_F(SensorsFixture, IGN_UTILS_TEST_DISABLED_ON_MAC(HandleRemovedEntities))
 
   gazebo::Server server(serverConfig);
 
+  common::ConnectionPtr postRenderConn;
+
   // A pointer to the ecm. This will be valid once we run the mock system
   gazebo::EntityComponentManager *ecm = nullptr;
   this->mockSystem->preUpdateCallback =
@@ -123,6 +185,15 @@ TEST_F(SensorsFixture, IGN_UTILS_TEST_DISABLED_ON_MAC(HandleRemovedEntities))
     {
       ecm = &_ecm;
     };
+  this->mockSystem->configureCallback =
+    [&](const gazebo::Entity &,
+           const std::shared_ptr<const sdf::Element> &,
+           gazebo::EntityComponentManager &,
+           gazebo::EventManager &_eventMgr)
+    {
+      postRenderConn = _eventMgr.Connect<gazebo::events::PostRender>(
+          std::bind(&::OnPostRender));
+    };
 
   server.AddSystem(this->systemPtr);
   server.Run(true, 50, false);
@@ -130,6 +201,10 @@ TEST_F(SensorsFixture, IGN_UTILS_TEST_DISABLED_ON_MAC(HandleRemovedEntities))
 
   testDefaultTopics();
 
+  testSensorEntityIds(*ecm, g_sensorEntityIds);
+  g_sensorEntityIds.clear();
+  g_scene.reset();
+
   // We won't use the event manager but it's required to create an
   // SdfEntityCreator
   gazebo::EventManager dummyEventMgr;
