@@ -22,6 +22,7 @@
 #include <ignition/common/Console.hh>
 #include <ignition/common/Util.hh>
 #include <ignition/transport/Node.hh>
+#include <ignition/utilities/ExtraTestMacros.hh>
 
 #include "ignition/gazebo/Link.hh"
 #include "ignition/gazebo/Model.hh"
@@ -42,19 +43,20 @@ class ThrusterTest : public InternalFixture<::testing::Test>
   /// \brief Test a world file
   /// \param[in] _world Path to world file
   /// \param[in] _namespace Namespace for topic
+  /// \param[in] _topic Thrust topic
   /// \param[in] _coefficient Thrust coefficient
   /// \param[in] _density Fluid density
   /// \param[in] _diameter Propeller diameter
   /// \param[in] _baseTol Base tolerance for most quantities
   public: void TestWorld(const std::string &_world,
-      const std::string &_namespace, double _coefficient, double _density,
-      double _diameter, double _baseTol);
+      const std::string &_namespace, const std::string &_topic,
+      double _coefficient, double _density, double _diameter, double _baseTol);
 };
 
 //////////////////////////////////////////////////
 void ThrusterTest::TestWorld(const std::string &_world,
-    const std::string &_namespace, double _coefficient, double _density,
-    double _diameter, double _baseTol)
+    const std::string &_namespace, const std::string &_topic,
+    double _coefficient, double _density, double _diameter, double _baseTol)
 {
   // Start server
   ServerConfig serverConfig;
@@ -121,8 +123,7 @@ void ThrusterTest::TestWorld(const std::string &_world,
 
   // Publish command and check that vehicle moved
   transport::Node node;
-  auto pub = node.Advertise<msgs::Double>(
-      "/model/" + _namespace + "/joint/propeller_joint/cmd_thrust");
+  auto pub = node.Advertise<msgs::Double>(_topic);
 
   int sleep{0};
   int maxSleep{30};
@@ -133,23 +134,44 @@ void ThrusterTest::TestWorld(const std::string &_world,
   EXPECT_LT(sleep, maxSleep);
   EXPECT_TRUE(pub.HasConnections());
 
-  double force{300.0};
+  // input force cmd - this should be capped to 0
+  double forceCmd{-1000.0};
   msgs::Double msg;
-  msg.set_data(force);
+  msg.set_data(forceCmd);
+  pub.Publish(msg);
+
+  // Check no movement
+  fixture.Server()->Run(true, 100, false);
+  std::this_thread::sleep_for(std::chrono::milliseconds(100));
+  EXPECT_DOUBLE_EQ(0.0, modelPoses.back().Pos().X());
+  EXPECT_EQ(100u, modelPoses.size());
+  EXPECT_EQ(100u, propellerAngVels.size());
+  modelPoses.clear();
+  propellerAngVels.clear();
+
+  // input force cmd this should be capped to 300
+  forceCmd = 1000.0;
+  msg.set_data(forceCmd);
   pub.Publish(msg);
 
   // Check movement
-  for (sleep = 0; modelPoses.back().Pos().X() < 5.0 && sleep < maxSleep;
-      ++sleep)
+  if (_namespace != "lowbattery")
   {
-    std::this_thread::sleep_for(std::chrono::milliseconds(100));
-    fixture.Server()->Run(true, 100, false);
-  }
-  EXPECT_LT(sleep, maxSleep);
-  EXPECT_LT(5.0, modelPoses.back().Pos().X());
+    for (sleep = 0; modelPoses.back().Pos().X() < 5.0 && sleep < maxSleep;
+        ++sleep)
+    {
+      std::this_thread::sleep_for(std::chrono::milliseconds(100));
+      fixture.Server()->Run(true, 100, false);
+    }
+    EXPECT_LT(sleep, maxSleep);
+    EXPECT_LT(5.0, modelPoses.back().Pos().X());
 
-  EXPECT_EQ(100u * sleep, modelPoses.size());
-  EXPECT_EQ(100u * sleep, propellerAngVels.size());
+    EXPECT_EQ(100u * sleep, modelPoses.size());
+    EXPECT_EQ(100u * sleep, propellerAngVels.size());
+  }
+
+  // max allowed force
+  double force{300.0};
 
   // F = m * a
   // s = a * t^2 / 2
@@ -159,6 +181,14 @@ void ThrusterTest::TestWorld(const std::string &_world,
   double xTol{1e-2};
   for (unsigned int i = 0; i < modelPoses.size(); ++i)
   {
+    if (_namespace == "lowbattery" && i > 545)
+    {
+      // Battery discharged should not accelerate
+      EXPECT_NEAR(modelPoses[i-1].Pos().X() - modelPoses[i-2].Pos().X(),
+        modelPoses[i].Pos().X() - modelPoses[i-1].Pos().X(), 1e-6);
+      continue;
+    }
+
     auto pose = modelPoses[i];
     auto time = dt * i;
     EXPECT_NEAR(force * time * time / (2 * mass), pose.Pos().X(), xTol);
@@ -169,7 +199,7 @@ void ThrusterTest::TestWorld(const std::string &_world,
 
     // The joint velocity command adds some roll to the body which the PID
     // wrench doesn't
-    if (_namespace == "custom")
+    if (_namespace == "custom" || _namespace == "lowbattery")
       EXPECT_NEAR(0.0, pose.Rot().Roll(), 0.1);
     else
       EXPECT_NEAR(0.0, pose.Rot().Roll(), _baseTol);
@@ -186,7 +216,14 @@ void ThrusterTest::TestWorld(const std::string &_world,
     // It takes a few iterations to reach the speed
     if (i > 25)
     {
-      EXPECT_NEAR(omega, angVel.X(), omegaTol) << i;
+      if (_namespace == "lowbattery" && i > 545)
+      {
+        EXPECT_NEAR(0.0, angVel.X(), _baseTol);
+      }
+      else
+      {
+        EXPECT_NEAR(omega, angVel.X(), omegaTol) << i;
+      }
     }
     EXPECT_NEAR(0.0, angVel.Y(), _baseTol);
     EXPECT_NEAR(0.0, angVel.Z(), _baseTol);
@@ -194,23 +231,42 @@ void ThrusterTest::TestWorld(const std::string &_world,
 }
 
 /////////////////////////////////////////////////
-TEST_F(ThrusterTest, PIDControl)
+// See https://github.com/ignitionrobotics/ign-gazebo/issues/1175
+TEST_F(ThrusterTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(PIDControl))
 {
+  const std::string ns{"sub"};
+  const std::string topic = "/model/" + ns +
+      "/joint/propeller_joint/cmd_thrust";
   auto world = common::joinPaths(std::string(PROJECT_SOURCE_PATH),
       "test", "worlds", "thruster_pid.sdf");
 
   // Tolerance could be lower (1e-6) if the joint pose had a precise 180
   // rotation
-  this->TestWorld(world, "sub", 0.004, 1000, 0.2, 1e-4);
+  this->TestWorld(world, ns, topic, 0.004, 1000, 0.2, 1e-4);
 }
 
 /////////////////////////////////////////////////
-TEST_F(ThrusterTest, VelocityControl)
+TEST_F(ThrusterTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(VelocityControl))
 {
+  const std::string ns = "custom";
+  const std::string topic = "/model/" + ns +
+      "/joint/propeller_joint/cmd_thrust";
   auto world = common::joinPaths(std::string(PROJECT_SOURCE_PATH),
       "test", "worlds", "thruster_vel_cmd.sdf");
 
   // Tolerance is high because the joint command disturbs the vehicle body
-  this->TestWorld(world, "custom", 0.005, 950, 0.25, 1e-2);
+  this->TestWorld(world, ns, topic, 0.005, 950, 0.25, 1e-2);
+}
+
+/////////////////////////////////////////////////
+TEST_F(ThrusterTest, IGN_UTILS_TEST_DISABLED_ON_WIN32(BatteryIntegration))
+{
+  const std::string ns = "lowbattery";
+  const std::string topic =  ns + "/thrust";
+  auto world = common::joinPaths(std::string(PROJECT_SOURCE_PATH),
+      "test", "worlds", "thruster_battery.sdf");
+
+  // Tolerance is high because the joint command disturbs the vehicle body
+  this->TestWorld(world, ns, topic, 0.005, 950, 0.25, 1e-2);
 }
 
